

#define DEBUG 1




#define MAXBANDS 8

  
#define GLCD_CL_BLACK 0x0000
#define GLCD_CL_WHITE 0xFFFF
#define GLCD_CL_GRAY 0x7BEF
#define GLCD_CL_LIGHT_GRAY 0xC618
#define GLCD_CL_GREEN 0x07E0
#define GLCD_CL_LIME 0x87E0
#define GLCD_CL_BLUE 0x001F
#define GLCD_CL_RED 0xF800
#define GLCD_CL_AQUA 0x5D1C
#define GLCD_CL_YELLOW 0xFFE0
#define GLCD_CL_MAGENTA 0xF81F
#define GLCD_CL_CYAN 0x07FF
#define GLCD_CL_DARK_CYAN 0x03EF
#define GLCD_CL_ORANGE 0xFCA0
#define GLCD_CL_PINK 0xF97F
#define GLCD_CL_BROWN 0x8200
#define GLCD_CL_VIOLET 0x9199
#define GLCD_CL_SILVER 0xA510
#define GLCD_CL_GOLD 0xA508
#define GLCD_CL_NAVY 0x000F
#define GLCD_CL_MAROON 0x7800
#define GLCD_CL_PURPLE 0x780F
#define GLCD_CL_OLIVE 0x7BE0



//   COFIGURATION PARAMETERS    //

#define IFx  2
#define IFy  150
#define IFwid  44
#define IFhi  30
#define IFlip  5
#define PBT_Incr IFwid


// Keyboard Modes
#define KEY_IDLE   0
#define KEY_FREQ   1
#define KEY_TONE   2
#define KEY_WPM    3
#define KEY_CW     4

// Keyboard Commands:
#define CTL_c   3    //CTL_c         Toggle CW Keyboard
#define CTL_d   4    //CTL_d         band down
#define CTL_f   6    //CTL_fxxxxxxxx set active vfo frequency
#define CTL_i   9    //CTL_i         toggle iambic 
#define CTL_l   12   //CTL_l         change lock
#define CTL_m   13   //CTL_m         change mode
#define CTL_p   16   //CTL_p         toggle paddle
#define CTL_r   18   //CTL_r         toggle RIT 
#define CTL_s   19   //CTL_s         toggle SPLIT
#define CTL_t   20   //CTL_txxx      set CW tone       100=< xxx <= 990
#define CTL_u   21   //CTL_u         band up
#define CTL_v   22   //CTL_v         toggle VFO
#define CTL_w   23   //CTL_wxx       set CW speed      5 =< xx <= 50
#define CTL_IF  14   //CTL_>         toggle IF shift

#define MAX_KEYING_BUFFER 255
#define MAXKYBDENTRY 24

/*

 *File name ubitx.h 
 *  The second set of 16 pins on the Raduino's bottom connector are have the three clock outputs and the digital lines to control the rig.
 */ 
#define V5_CLOCK_MINIMUM     500000l
#define LEGACY_CLOCK_MINIMUM 400000l


/**
 * These are the indices where these user changeable settings are stored  in the EEPROM
 */
#define MASTER_CAL   0
#define LSB_CAL      4    // NOT USED
#define USB_CAL      8
#define SIDE_TONE   12
//these are ID's of the VFO's as well as their offset into the EEPROM storage, don't change these 'magic' values
#define VFO_A       16
#define VFO_B       20
#define CW_SIDETONE 24
#define CW_SPEED    28


//  Added EEPROM values

#define FREQ_INCR            32
#define SELECTED_MEM_TABLE   36 
#define MODE                 40
#define VOLTS                44
#define BAUDD                48
#define XTALPF               52
#define IAMBIC_MODE          56
#define PDL_SWAP_VAL         60
#define VOLT_DIVIDE          64
#define SMETER_ACTIVE        68
#define CLOCK_ACTIVE         72
#define CW_FILTER_PRESENT    76
#define CW_FILTER_CALIBRATED 80
#define CW_FILTER_OFFSET     84
#define BOARD_VERSION        88
#define V5_MASTER_CAL        92
#define V5_USB_CAL           96


/**
 * The uBITX is an upconversion transceiver. The first IF is at 45 MHz.
 * The first IF frequency is not exactly at 45 MHz but about 5 khz lower,
 * this shift is due to the loading on the 45 MHz crystal filter by the matching
 * L-network used on it's either sides.
 * The first oscillator works between 48 MHz and 75 MHz. The signal is subtracted
 * from the first oscillator to arrive at 45 MHz IF. Thus, it is inverted : LSB becomes USB
 * and USB becomes LSB.
 * The second IF of 12 MHz has a ladder crystal filter. If a second oscillator is used at 
 * 57 MHz, the signal is subtracted FROM the oscillator, inverting a second time, and arrives 
 * at the 12 MHz ladder filter thus double inversion, keeps the sidebands as they originally were.
 * If the second oscillator is at 33 MHz, the oscillator is subtracted from the signal, 
 * thus keeping the signal's sidebands inverted. The USB will become LSB.
 * We use this technique to switch sidebands. This is to avoid placing the lsbCarrier close to
 * 12 MHz where its fifth harmonic beats with the Arduino's 16 MHz oscillator's fourth harmonic
 */

// the second oscillator should ideally be at 57 MHz, however, the crystal filter's center frequency 
// is shifted down a little due to the loading from the impedance matching L-networks on either sides
#define SECOND_OSC_USB (56995000l)
#define V5_SECOND_OSC_USB (45005000L)   //56995000l
#define SECOND_OSC_LSB (32995000l) 
//these are the two default USB and LSB frequencies. The best frequencies depend upon your individual taste and filter shape
#define INIT_USB_FREQ   (11996500l)
// limits the tuning and working range of the uBITX between 3 MHz and 30 MHz
#define LOWEST_FREQ  (3000000l)
#define HIGHEST_FREQ (30000000l)

//we directly generate the CW by programming the Si5351 to the cw tx frequency, hence, both are different modes
//these are the parameters passed to startTx
#define TX_SSB 0
#define TX_CW 1

#define CW_TIMEOUT (350l) 
#define PTT_HNDKEY_DEBOUNCE_CT 2



#define MODE_USB  0
#define MODE_LSB  1
#define MODE_CW   2
#define MODE_CWR  3
#define MODE_SWU  4
#define MODE_SWL  5
#define MODE_CWP  6


#define BUTTON_DELAY_1 40
#define BUTTON_DELAY_2 120
#define BUTTON_DELAY_3 260
#define BUTTON_DELAY_4 360

/////////////////////////  keyer

#define DIT_L 0x01 // DIT latch
#define DAH_L 0x02 // DAH latch
#define DIT_PROC 0x04 // DIT is being processed
enum KSTYPE {IDLE, CHK_DIT, CHK_DAH, KEYED_PREP, KEYED, INTER_ELEMENT };

//////////////////////////////////////////////


#define PRESS_QUICK  80
#define PRESS_MEDIUM 160
#define PRESS_LONG   240
#define PRESS_LONGER   320
#define PRESS_LONGEST  400
#define ENCODER_FREQ_SHIFT 20


//////////////////////////////////////////////////////   PIN DEFINITONS12345789

  #define TX_RX (10)
  #define CW_TONE (9)
  #define TX_LPF_A (8)
  #define TX_LPF_B (7)
  #define TX_LPF_C (6)
  #define CW_KEY (5)




  #define ENC_A (19)
  #define ENC_B (18)
  #define FBUTTON (20)
  #define UBIT_VCC (A13)
  #define DIGITAL_DOT  (22)
  #define DIGITAL_DASH (23)
  #define DIGITAL_KEY (21)
  #define PTT   (21)

  #define SMETERREADPIN (A19)
