// File name ubitx_Configuration.cpp


#include <Arduino.h>
#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <XPT2046_Touchscreen.h>
#include <SPIN.h>
#include <SD.h>
#include <TimeLib.h>
#include "ubitx.h"

extern ILI9341_t3n tft;

extern void Get_Touch( bool wait );
void Init_Screen();
extern void updateDisplay(void);
extern void Display_RIT(void);
extern bool Set_Left_Button_Pressed( void );
extern bool Get_Left_Button_Pressed( void );
extern void   Get_Mouse_Data( void );
extern int Get_Mouse_Wheel_Value(void);
extern void   Save_Mouse( void );
extern void Display_Mouse( void);
extern void Fake_Delay(void);
extern void Process_Board_Version( void );

extern TS_Point p;
extern volatile int LastSmeter;
extern bool Menu_Changed;
extern int mouse_x;
extern int mouse_y;
extern int last_mouse_x;
extern int last_mouse_y;
extern int Board_Version;

extern void Process_Reboot();
extern void si5351_set_calibration(int32_t cal);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern int  enc_read(void);
extern void initOscillators(void);
extern void setFrequency(unsigned long f);
extern void ritDisable(void);
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );

extern unsigned long usbCarrier;
extern char c[21], b[21];
extern unsigned long frequency;
extern volatile int ubitx_mode;





unsigned long Cal_Freq_Incr[7] = {1l, 10l, 100l, 1000l, 10000l, 100000l, 1000000l};
int Cal_Freq_Index = 0;

int real_index[7] = { 270, 256, 242, 228, 214, 200, 186 };

extern XPT2046_Touchscreen ts;

extern int32_t calibration;
extern uint32_t si5351bx_vcoa;


void printCarrierFreq(unsigned long freq) {

  memset(b, 0, sizeof(b));

  sprintf(b, "%08d", (int)freq);
  tft.setFont(Arial_18);
  tft.setTextColor(ILI9341_YELLOW);
  tft.fillRect(172, 180, 120, 20, ILI9341_BLUE);
  tft.setCursor(172, 180);
  tft.print(b);
  if ( (p.x > 172) && (p.x < 292) && (p.y > 180) && (p.y < 200)) Save_Mouse();
}






void Update_Freq(void) {

  tft.setFont(Arial_18);
  tft.setTextColor(ILI9341_YELLOW);
  tft.setCursor(10, 180);
  tft.print((char *)"Tuned: LSB  ");
  printCarrierFreq(frequency);

}

void CaldoTuning() {
  int s;

  s = enc_read();
  if ( s == 0 ) {
    s = Get_Mouse_Wheel_Value();
  }
  if (s) {

    if (s > 4)
      frequency += Cal_Freq_Incr[Cal_Freq_Index];
    else if (s >  2)
      frequency += Cal_Freq_Incr[Cal_Freq_Index];
    else if (s >  0)
      frequency +=  Cal_Freq_Incr[Cal_Freq_Index];
    else if (s <  0)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    else if (s < -2)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    else if (s < -4)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    if ( frequency < 1000000) frequency = 1000000;

    setFrequency(frequency);
    Update_Freq();
  }
}



int menuSetupCalibration( void ) {
  int knob = 0;



  calibration = 0;
  si5351_set_calibration(calibration);
  setFrequency(frequency);
  tft.fillScreen(ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(10, 20);
  tft.print((char *)"Step 2: Best Sound");
  tft.setFont(Arial_14);
  tft.setCursor(10, 50);
  tft.print((char *)"EXIT: Short Encoder button");
  tft.setCursor(10, 70);
  tft.print((char *)"EXIT: Short  Left Mouse button");
  strcpy(b, "Zero-beat cal: ");
  ltoa(calibration / 8750, c, 10);
  strcat(b, c);
  tft.setFont(Arial_18);
  tft.setTextColor(ILI9341_WHITE);

  tft.setCursor(10, 180);
  tft.print(b);
  Display_Mouse();

  while ((digitalRead(FBUTTON) == HIGH ) && (Set_Left_Button_Pressed() == false) ) {

    knob = enc_read();
    if ( knob == 0 )knob = Get_Mouse_Wheel_Value();

    if (knob > 0) {
      calibration += 8750;
      usbCarrier += 120;
    } else if (knob < 0) {
      calibration -= 8750;
      usbCarrier -= 120;
    } else continue; //don't update the frequency or the display

    si5351_set_calibration(calibration);
    si5351bx_setfreq(0, usbCarrier);
    setFrequency(frequency);

    strcpy(b, "Zero-beat cal: ");
    ltoa(calibration / 8750, c, 10);
    strcat(b, c);
    tft.fillRect(10, 180, 200, 18, ILI9341_BLUE);
    tft.setCursor(10, 180);
    tft.print(b);
    if ( (p.x > 10) && (p.x < 210) && (p.y > 180) && (p.y < 198)) Save_Mouse();
  }

  //save the setting

  tft.fillScreen(ILI9341_BLUE);
  tft.setFont(Arial_16);
  tft.setTextColor(ILI9341_RED, ILI9341_WHITE);
  tft.setCursor(100, 120);
  tft.print((char *)"Calibration set!");
  tft.setTextColor(ILI9341_WHITE, ILI9341_RED);
  tft.setCursor(100, 180);
  tft.print((char *)"Set Carrier now");
  if ( Board_Version == 5 )  Put_SDEEPROM( V5_MASTER_CAL, calibration );
  else Put_SDEEPROM( MASTER_CAL, calibration );
  delay(3000);

  initOscillators();
  si5351_set_calibration(calibration);
  setFrequency(frequency);
  while ( digitalRead(PTT) == LOW ) delay(10);
  while ( Set_Left_Button_Pressed() == true ) delay(10);
  return (0);
}




void DoCarrierSetup(void) {
  int knob = 0;
  int i;
  int looper = 1;


  tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
  tft.drawFastHLine(172, 203, 110, ILI9341_BLACK);
  tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
  tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
  si5351bx_setfreq(0, usbCarrier);
  printCarrierFreq(usbCarrier);

  while ( looper == 1 ) {
    knob = enc_read();
    if ( knob == 0 )knob = Get_Mouse_Wheel_Value();

    if (knob > 0) {
      usbCarrier += Cal_Freq_Incr[Cal_Freq_Index];
      printCarrierFreq(usbCarrier);
      si5351bx_setfreq(0, usbCarrier);
    } else if (knob < 0) {
      usbCarrier -= Cal_Freq_Incr[Cal_Freq_Index];
      printCarrierFreq(usbCarrier);
      si5351bx_setfreq(0, usbCarrier);
    }
    delay(100);
    //only if the button is pressed
    if (digitalRead(FBUTTON) == LOW) {
      delay(50);
      if (digitalRead(FBUTTON) == LOW) {
        // OK button is definitely pushed so now detect how long it is pressed.
        i = 0;
        while ( digitalRead(FBUTTON) == LOW ) {
          delay(10);
          i++;
          if (i > BUTTON_DELAY_2 ) {
            tft.setFont(Arial_18);
            tft.setTextColor(ILI9341_RED, ILI9341_WHITE);
            tft.setCursor(10, 200);
            tft.print((char *)"Carrier set!");

            looper = 0;
          }
        }
        if (i <= BUTTON_DELAY_2 ) {
          Cal_Freq_Index = Cal_Freq_Index - 1;
          if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
          tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
          tft.drawFastHLine(172, 203, 110, ILI9341_BLACK);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
          printCarrierFreq(usbCarrier);
        }
      }
    }
    if (ts.touched()) {
      Get_Touch( true );
      if ( (p.x > 15) && (p.x < 83) && (p.y > 140) && (p.y < 164))  looper = -1;
    }
    if ( Set_Left_Button_Pressed() == 1) {
      if ( (p.x > 15) && (p.x < 83) && (p.y > 140) && (p.y < 164)) {
        while ( Set_Left_Button_Pressed() == 1) Fake_Delay();
        looper = -1;
      } else {
        // OK button is definitely pushed so now detect how long it is pressed.
        i = 0;
        while ( Set_Left_Button_Pressed() == 1 ) {
          delay(10);
          i++;
          if (i > BUTTON_DELAY_2 ) {
            tft.setFont(Arial_18);
            tft.setTextColor(ILI9341_RED, ILI9341_WHITE);
            tft.setCursor(10, 200);
            tft.print((char *)"Carrier set!");

            looper = 0;
          }
        }
        if (i <= BUTTON_DELAY_2 ) {
          Cal_Freq_Index = Cal_Freq_Index - 1;
          if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
          tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
          tft.drawFastHLine(172, 203, 110, ILI9341_BLACK);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
          printCarrierFreq(usbCarrier);
        }
      }
    }
    //save the setting
    if ( looper == 0 ) {
      if ( Board_Version == 5 ) Put_SDEEPROM( V5_USB_CAL, usbCarrier );
      else Put_SDEEPROM( USB_CAL, usbCarrier );
      delay(1000);
      si5351bx_setfreq(0, usbCarrier);
      setFrequency(frequency);
    }
  }
}

void menuSetupCarrier(void) {
  // unsigned long prevCarrier;

  //keep clear of any previous button press
  while (digitalRead(FBUTTON) == LOW)
    delay(50);
  delay(50);

  // prevCarrier = usbCarrier;
  tft.fillScreen(ILI9341_BLACK);
  tft.setFont(Arial_18);
  tft.setTextColor(ILI9341_ORANGE);
  tft.setCursor(10, 10);
  tft.print((char *)"Step 3: Best Signal");
  tft.setFont(Arial_12);
  tft.setCursor(10, 60);
  tft.print((char *)"Long Encoder button to Exit");
  tft.setCursor(10, 80);
  tft.print((char *)"Short Encoder button for Freq. Step");
  usbCarrier = 11995000l;

  DoCarrierSetup();
}

void Process_BFO(void) {

  //keep clear of any previous button press
  while (digitalRead(FBUTTON) == LOW)
    delay(50);
  delay(50);


  //Get the setting
  if ( Board_Version == 5 ) usbCarrier = Get_SDEEPROM( V5_USB_CAL );
  else usbCarrier = Get_SDEEPROM( USB_CAL );
  tft.fillScreen(ILI9341_BLUE);
  tft.setFont(Arial_18_Bold);
  tft.fillRect(15, 140, 68, 26, ILI9341_RED);
  tft.setCursor(19, 144);
  tft.setTextColor(ILI9341_WHITE);
  tft.print((char *)"QUIT");
  tft.setFont(Arial_18);

  tft.setCursor(10, 10);
  tft.print((char *)"Adjust BFO");
  tft.setFont(Arial_14);
  tft.setCursor(10, 50);
  tft.print((char *)"Exit: Long Encoder button");
  tft.setCursor(10, 70);
  tft.print((char *)"Exit: Long  Left Mouse button");
  tft.setCursor(10, 100);
  tft.print((char *)"Freq Inc: Short Encoder button");
  tft.setCursor(10, 120);
  tft.print((char *)"Freq Inc: Short Left Mouse button");
  if ( !((p.x > 172) && (p.x < 292) && (p.y > 180) && (p.y < 200))) Save_Mouse();
  DoCarrierSetup();
  Menu_Changed = true;

}


void Menu_Calibrate(void) {
  int i;
  int looper = 1;
  bool Displayed = false;

  tft.fillScreen(ILI9341_BLUE);
  frequency = 10000000l;
  tft.setFont(Arial_18_Bold);
  tft.fillRect(15, 140, 68, 26, ILI9341_RED);
  tft.setCursor(19, 144);
  tft.setTextColor(ILI9341_WHITE);
  tft.print((char *)"QUIT");
  tft.setFont(Arial_18);

  tft.setCursor(10, 20);
  tft.print((char *)"Step 1:Tune station");
  tft.setFont(Arial_14);
  tft.setCursor(10, 50);
  tft.print((char *)"Exit: Long Encoder button");
  tft.setCursor(10, 70);
  tft.print((char *)"Exit: Long  Left Mouse button");
  tft.setCursor(10, 100);
  tft.print((char *)"Freq Inc: Short Encoder button");
  tft.setCursor(10, 120);
  tft.print((char *)"Freq Inc: Short Left Mouse button");
  ubitx_mode = MODE_LSB;
  ritDisable();
  Update_Freq();
  tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
  tft.drawFastHLine(172, 203, 110, ILI9341_BLACK);
  tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
  tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
  Save_Mouse();

  while ( looper == 1 ) {
    //only if the button is pressed
    if (digitalRead(FBUTTON) == LOW) {
      delay(50);
      if (digitalRead(FBUTTON) == LOW) {
        // OK button is definitely pushed so now detect how long it is pressed.
        i = 0;
        while ( digitalRead(FBUTTON) == LOW ) {
          delay(10);
          i++;
          if (i > BUTTON_DELAY_2 ) {
            if ( !Displayed ) {
              Displayed = true;
              tft.fillScreen(ILI9341_BLUE);
              tft.setCursor(10, 20);
              tft.print((char *)"Step 2: Best Sound");
            }
          }
        }
        if (i <= BUTTON_DELAY_2 ) {
          Cal_Freq_Index = Cal_Freq_Index - 1;
          if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
          tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
          tft.drawFastHLine(172, 203, 110, ILI9341_BLACK);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
          Update_Freq();
        } else looper = false;
      }
    }
    if ( Get_Left_Button_Pressed() == true) {
      if ( (p.x > 15) && (p.x < 83) && (p.y > 140) && (p.y < 164)) {
        while ( Set_Left_Button_Pressed() == 1) Fake_Delay();
        looper = -1;
      } else {
        i = 0;
        while ( Set_Left_Button_Pressed() == true ) {
          delay(10);
          i++;
          if (i > BUTTON_DELAY_2 ) {
            if ( !Displayed ) {
              Displayed = true;
              tft.fillScreen(ILI9341_BLUE);
              tft.setCursor(10, 20);
              tft.print((char *)"Step 2: Best Sound");
            }
          }
        }
        if (i <= BUTTON_DELAY_2 ) {
          Cal_Freq_Index = Cal_Freq_Index - 1;
          if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
          tft.drawFastHLine(172, 202, 110, ILI9341_BLACK);
          tft.drawFastHLine(172, 203, 111, ILI9341_BLACK);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 202, 12, ILI9341_GREEN);
          tft.drawFastHLine(real_index[Cal_Freq_Index], 203, 12, ILI9341_GREEN);
          Update_Freq();
        } else looper = false;

      }
    }
    if (ts.touched()) {
      Get_Touch( true );
      if ( (p.x > 15) && (p.x < 83) && (p.y > 140) && (p.y < 164))  looper = -1;
    }
    CaldoTuning();
    delay(100);
  }
  if ( looper == 0 ) menuSetupCalibration();
  if ( looper == -1 ) Display_Mouse();
  Menu_Changed = true;
}





void Display_Calibration_Menu( void) {
  tft.fillScreen(ILI9341_BLUE);
  tft.drawRect(138, 100, 172, 120,  ILI9341_WHITE);
  tft.fillRect(139, 101, 170, 118, ILI9341_BLUE);
  tft.setFont(Arial_16);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(141, 108);
  tft.print((char*)"BOARD ( V");
  if( Board_Version == 5 ) tft.print((char*)"5");
  else  tft.print((char*)"L");
  tft.print((char*)" )");
 
  tft.setCursor(141, 138);
  tft.println((char*)"CALIBRATE");
  tft.setCursor(141, 168);
  tft.println((char*)"BFO Adjust");
  tft.setCursor(141, 198);
  tft.setFont(Arial_16_Bold);
  tft.println((char*)"EXIT");
  Save_Mouse();


}



void Process_Board( void ){

  if( Board_Version == 5 ) Board_Version = 4;
  else  Board_Version = 5;
  Process_Board_Version();
  Display_Calibration_Menu();
}



void Do_Calibrate_Radio(void) {
  bool looper = true;

  while ( Set_Left_Button_Pressed() == 1) Fake_Delay();

  Display_Calibration_Menu();

  if ( Board_Version == 5 ) {
    calibration = 92;
    usbCarrier  = 96;
  } else {
    calibration = 0;
    usbCarrier  = 8;
  }

  while ( looper ) {
    if (ts.touched()) {
      Get_Touch(true);
      if ( (p.x > 140) && (p.x < 300) && (p.y > 137) && (p.y < 158)) Menu_Calibrate();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 164) && (p.y < 188)) Process_BFO();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 100) && (p.y < 130)) Process_Board();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 198) && (p.y < 218)) looper = false;

    }
    if ( Set_Left_Button_Pressed() == 1) {
      while ( Set_Left_Button_Pressed() == 1) Fake_Delay();

      if ( (p.x > 140) && (p.x < 300) && (p.y > 137) && (p.y < 158)) Menu_Calibrate();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 164) && (p.y < 188)) Process_BFO();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 100) && (p.y < 130)) Process_Board();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 198) && (p.y < 218)) looper = false;
    }
    if ( Menu_Changed == true ) {
      Menu_Changed = false;
      Display_Calibration_Menu();

    }
  }
  tft.fillScreen(ILI9341_BLACK);
  LastSmeter = 0;
  Process_Board_Version();
  
}
