
char Version_Text[21] = "Teensy_991.V2.01b";
char Version_Date[21] = "05/05/2019 23:30";

/**
   This source file is under General Public License version 3.

   This version changes CAT support to YAESU FT-991

   This version supports Legacy (V1-V4) and V5 uBITX boards.

   STARTING WITH THIS VERSION 2.8" and  3.2" Color TFT Display supported

   Added Pass Band Tuning (PBT):
   Acknowledging Jerry Gaffke (KE7ER) and Tim Gorman (AB0WR) for their descriptions on how PBT works
   from the BITX20 group.


   This version uses a built-in Si5351 library
   Most source code are meant to be understood by the compilers and the computers.
   Code that has to be hackable needs to be well understood and properly documented.
   Donald Knuth coined the term Literate Programming to indicate code that is written be
   easily read and understood.



   The system starts executing the code in a function called setup() and then it
   repeatedly keeps calling loop() forever. All the initialization code is kept in setup()
   and code to continuously sense the tuning knob, the function button, transmit/receive,
   etc is all in the loop() function. If you wish to study the code top down, then scroll
   to the bottom of this file and read your way up.


    The main chip which generates up to three oscillators of various frequencies in the
    Raduino is the Si5351a. To learn more about Si5351a you can download the datasheet
    from www.silabs.com although, strictly speaking it is not a requirement to understand this code.
    Instead, you can look up the Si5351 library written by xxx, yyy. You can download and
    install it from www.url.com to compile this file.
    The Wire.h library is used to talk to the Si5351 and we also declare an instance of
    Si5351 object to control the clocks.
*/

/*
    1. Renamed all files with .cpp extension except ubitx20.ino
    2. Creates ubitx20.h to hold constants that are used by multiple files.
    3. Created list of external routines and variable not defined in current file.

    This will be the base for further development.

    Ron - W2CTX
    Jim - W0EB (editor & beta tester)
*/


#include <Arduino.h>

#include <i2c_t3.h>
#include "USBHost_t36.h"

#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <SPIN.h>
#include <XPT2046_Touchscreen.h>
#include <SPI.h>
#include <SD.h>
#include <EEPROM.h>
#include <TimeLib.h>
#include "ubitx.h"


extern void Process_Mouse(void);
extern void Process_Buttons(void);
extern void Update_Mouse(void);
extern void Remember_Mouse(void);
extern void Display_Mouse(void);
extern void Remove_Mouse(void);
extern void Get_Touch( bool wait  );
extern bool Set_Left_Button_Pressed( void );
extern void Save_Mouse(void);
extern void   Get_Mouse_Data( void );

extern TS_Point p;

extern void tftPrint(int x, int y, char* Text, ILI9341_t3_font_t, int fontColor, bool textframed, int frameColor );
extern void Place_The_Cursor(void);
extern void Display_RIT_Data(void);
extern void menuSetupCwTone();
extern void menuCWSpeed();
extern void Display_Table_Menu();
extern void Remove_Table_Menu(void);
extern void Display_Channel_Menu();
extern void Remove_Channel_Menu(void);
extern void Calibrate_Screen( void );
extern void Read_Screen_CAL_to_SD( void );
extern void Display_Trangle(int xstart, int ystart, int width, int height, uint16_t color);

extern void Read_Band_Table_from_SD(void);
extern void Write_Entire_Band_Table_to_SD(void);
extern void Write_Band_Table_Update_to_SD();
extern void Do_Mouse_Configuration1(void);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void updateDisplay(void);
extern int  btnDown(void);
extern int  enc_read(void);
extern void initOscillators(void);
extern void cwKeyer(void);
extern void menuVfoToggle(void );
extern void Connect_Interrupts(void);
extern void Init_Cat( long baud );
extern void Process_CAT_Command( void );
extern void Do_CW_Options(void);
extern void Do_Mouse_CW_Options(void);
extern void Do_Mouse_Configuration2();
extern void Do_Configuration1(void);
extern void Do_Configuration2(void);
extern void Do_Calibrate_Radio(void);
extern void DoBaud(void);
extern void menuPaddleSwapToggle( void );
extern void SetIambicA( void );
extern void SetIambicB( void );
extern void SetPaddleSwapN( void);
extern void SetPaddleSwapR( void);
extern void Place_The_Cursor( void);
extern void Display_Voltage( void );
extern void Display_S_Meter();
extern bool Choose_Down(void);
extern bool Enter_Exit_Down(void);
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );
extern void SD_Setup();
extern void Display_Mem_Selection( void );
extern int Menu_Select_Encoder_Read(void);
extern void Display_S_Meter( void );
extern void Process_Keyboard( void );
extern void Process_CW( void );
extern void Display_Mode( void);
extern unsigned long Clock_Minimum;

void startTx(byte txMode);
void stopTx(void);

extern int Last_Mode;
extern int isUSB;
extern long ubitx_CAT_Long_Baud_rates[8];
extern int ubitx_baud;
extern int Last_Baud;
extern unsigned int SI5351BX_XTALPF;              // 1:6pf  2:8pf  3:10pf
extern bool Cat_Lock;
extern int Memory_Table_Select;
extern unsigned long Working_Copy_Frequencies[10];
extern int   Working_Copy_Modes[10];
extern int TableMODES[4][10];
extern unsigned long Band_Table[2][8];
extern unsigned long Clock_Minimum;

//extern bool CW_Filter_Present;
extern bool CW_Calibrated;
//extern unsigned long CW_Filter_Offset;
extern unsigned long Standard_Offset;

extern int Mem_Number;
extern bool Choose_Down(void);



extern time_t pctime;
extern volatile int SmeterReadValue, LastSmeter, SMeter_Value, smeterInterval;
extern int CAL_Minx;
extern int CAL_Miny;
extern int CAL_Maxx;
extern int CAL_Maxy;

extern int16_t mouse_x;
extern int16_t mouse_y;
extern int16_t last_mouse_x;
extern int16_t last_mouse_y;

extern int mouse_left_button;
extern int mouse_right_button;
extern int Current_Mouse_Wheel;

extern uint16_t Mouse_Data[200];

extern bool Mouse_Active;

int Board_Version;

extern  File fileptr;

/**
    The main chip which generates up to three oscillators of various frequencies in the
    Raduino is the Si5351a. To learn more about Si5351a you can download the datasheet
    from www.silabs.com although, strictly speaking it is not a requirement to understand this code.

    We no longer use the standard SI5351 library because of its huge overhead due to many unused
    features consuming a lot of program space. Instead of depending on an external library we now use
    Jerry Gaffke's, KE7ER, lightweight standalone minimalist "si5351bx" routines (see further down the
    code). Here are some defines and declarations used by Jerry's routines:
*/




//You are using 4 wire SPI here, so:
//SPI0:
#define SPI_DC    15
#define SPI_MOSI  11
#define SPI_MISO  12
#define SPI_SCK   13
#define SPI_CS    14

#define TS_CS   16
#define TS_IRQ  17

ILI9341_t3n tft = ILI9341_t3n( SPI_CS, SPI_DC, 255, SPI_MOSI, SPI_SCK, SPI_MISO, &SPIN);


XPT2046_Touchscreen ts(TS_CS, TS_IRQ);


//////////////////////////////////////////////
extern bool Key_Displayed;


/*
  The assumption is that there is:
    single keyboard connected
    single mouse connected
    combo keyboard/mouse connected
    or nothing
*/
USBHost uBITX_USB;
USBHub hub1(uBITX_USB);
USBHub hub2(uBITX_USB);
KeyboardController uBITX_Keyboard(uBITX_USB);
USBHIDParser hid1(uBITX_USB);
USBHIDParser hid2(uBITX_USB);
USBHIDParser hid3(uBITX_USB);
USBHIDParser hid4(uBITX_USB);
USBHIDParser hid5(uBITX_USB);
MouseController uBITX_Mouse(uBITX_USB);
JoystickController uBITX_Joystick(uBITX_USB);

USBDriver *drivers[] = {&hub1, &hub2, &uBITX_Keyboard, &hid1, &hid2};
#define CNT_DEVICES (sizeof(drivers)/sizeof(drivers[0]))
const char * driver_names[CNT_DEVICES] = {"Hub1", "Hub2", "KB1", "HID1", "HID2"};
bool driver_active[CNT_DEVICES] = {false, false, false, false, false};

// Lets also look at HID Input devices
USBHIDInput *hiddrivers[] = {&uBITX_Mouse};
#define CNT_HIDDEVICES (sizeof(hiddrivers)/sizeof(hiddrivers[0]))
const char * hid_driver_names[CNT_DEVICES] = {"uBITX_Mouse" };
bool hid_driver_active[CNT_DEVICES] = {false };




extern int Keyboard_Mode;
extern int KeyIndex;
char Keyboard_Input_Buffer[41];


unsigned long Ubitx_Voltage = 0;
unsigned long Current_Ubitx_Voltage = 0;
unsigned long Converted_Ubitx_Voltage = 0;

char Voltage_Text[5];
unsigned long Volt_Divider;

volatile int ubitx_mode;

extern int wpm;
bool Clock_Active = false;
bool Smeter_Active = false;
bool Volts_Active = false;
int Volts_Delay_Ctr = 0;
int Current_Band = 0;
bool Freq_Incr_Direction = false;
bool Tune_Lock = false;
bool Set_Time = false;

long PBT_Value = 0;        //   OFF = 0;
//bool PBT_Active = false;    // true = encoder connected;
bool PBT_ON = false;        // PBT ON or OFF


/////  Band

int VFOA_BAND;
int VFOB_BAND;




//////////////////// keyer
unsigned volatile char keyerControl = 0;
unsigned volatile char keyerState = 0;
unsigned volatile char IAMBIC = 0x10;  // 0 for Iambic A, 1 for Iambic B
unsigned volatile char PDLSWAP =   0x00;     // 0x00 for normal, 0x08 for swap

///////////////////

char c[21], b[21];
bool ritOn = false;
//bool Rit_Encoder = false;
//bool PBT_Encoder = false;
char splitOn = 0;
char vfoActive = VFO_A;
unsigned long vfoA = 07150000L, vfoB = 14200000L, sideTone = 000, usbCarrier;
unsigned long Last_New_Frequency = 0;
int Last_Frequency_Save_Ctr = 0;
unsigned long frequency, ritRxFrequency, ritTxFrequency;  //frequency is the current frequency on the dial
long ritOffset = 0;
long LastritOffset = 0;
long ritOffsetMax = 5000;
unsigned long Freq_Incr[6] = {10, 100, 1000, 10000, 100000, 1000000};
int Freq_Index = 0;

int Freq_Underline[6] = {19, 18, 16, 15, 14, 12};
int Menu_Index = 0;  // new menu

int cwSpeed; //this is actual the dot period in milliseconds
bool CW_Active;

extern int32_t calibration;

/**
   Raduino needs to keep track of current state of the transceiver. These are a few variables that do it
*/
volatile bool inTx = false;                //it is set to 1 if in transmit mode (whatever the reason : CW, PTT or CAT)

byte menuOn = 0;              //set to 1 when the menu is being displayed, if a menu item sets it to zero, the menu is exited
unsigned char txFilter = 0;   //which of the four transmit filters are in use
boolean modeCalibrate = false;//this mode of menus shows extended menus to calibrate the oscillators and choose the proper


void ritEnable(void);
void ritDisable(void);



void Process_Memory_Load( void);
///////////////////////////////////////////// GRAPHIC CODE /////////////////////////////////////////////

void Pop_Up_Calibration_Notice( void ) {

  //tft.drawRect(10, 106, 280, 80,  ILI9341_YELLOW);
  tft.fillRect(11, 107, 278, 78,  ILI9341_YELLOW);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(14, 120);
  tft.setTextColor(ILI9341_RED);
  tft.println("Calibration bad: default value set");
  tft.setCursor(14, 140);
  tft.print("MUST CALIBRATE uBITX");
  tft.setCursor(14, 160);
  tft.print("Cycle power to exit");
  while ( 1 == 1 )delay(5000);

}



void Display_RX(void) {
  tft.fillRect(64, 124, 28, 22,  ILI9341_BLACK);
  tft.drawRect(64, 126, 28, 18,  ILI9341_WHITE);
  tft.fillRect(65, 127, 26, 16,  ILI9341_GREEN);
  tft.setFont(Arial_14);
  tft.setCursor(66, 128);
  tft.setTextColor(ILI9341_BLACK);
  tft.print("Rx");
}

void Display_TX(void) {
  tft.drawRect(64, 124, 28, 22,  ILI9341_WHITE);
  tft.fillRect(65, 125, 26, 20,  ILI9341_RED);
  tft.setFont(Arial_14);
  tft.setCursor(66, 128);
  tft.setTextColor(ILI9341_WHITE);
  tft.print("Tx");
}

void Display_SPL(void) {
  tft.drawRect(50, 196, 40, 23,  ILI9341_WHITE);
  tft.fillRect(51, 197, 38, 21,  ILI9341_BLUE);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(54, 201);
  tft.setTextColor(ILI9341_WHITE);
  tft.print("SPL");
  splitOn = 0;
}


void Display_SPL_ON(void) {
  tft.drawRect(50, 196, 40, 23,  ILI9341_WHITE);
  tft.fillRect(51, 197, 38, 21,  ILI9341_GREEN);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(54, 201);
  tft.setTextColor(ILI9341_BLACK);
  tft.print("SPL");
  splitOn = 1;
}

void Display_Lock( void) {
  tft.drawRect(2, 196, 44, 23,  ILI9341_WHITE);
  tft.fillRect(3, 197, 42, 21,  ILI9341_BLUE);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(5, 201);
  tft.setTextColor(ILI9341_WHITE);
  tft.print("Lock");
  Tune_Lock = false;
}


void Display_Lock_ON( void) {
  tft.drawRect(2, 196, 44, 23,  ILI9341_WHITE);
  tft.fillRect(3, 197, 42, 21,  ILI9341_RED);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(5, 201);
  tft.setTextColor(ILI9341_WHITE);
  tft.print("Lock");
  Tune_Lock = true;
}

void Display_Setup( void) {
  tft.drawRect(138, 196, 54, 23,  ILI9341_WHITE);
  tft.fillRect(139, 197, 52, 21,  ILI9341_BLUE);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(142, 201);
  tft.setTextColor(ILI9341_WHITE);
  tft.print("Setup");
}

void Display_RIT(void) {
//  if ( ritOn && (Rit_Encoder == false) ) {
//    PBT_Active = false;
//    Rit_Encoder = true;
//  } else {
    tft.drawRect(94, 196, 40, 23,  ILI9341_WHITE);
    tft.fillRect(95, 197, 38, 21,  ILI9341_BLUE);
    tft.setFont(Arial_12_Bold);
    tft.setCursor(100, 201);
    tft.setTextColor(ILI9341_WHITE);
    tft.print("RIT");
    tft.fillRect(196, 190, 800, 32,  ILI9341_BLACK);
    ritOn = false;
//    Rit_Encoder = false;
    ritDisable();
 //   Update_Mouse();
//  }
}


void Display_RIT_ON(void) {
  tft.drawRect(94, 196, 40, 23,  ILI9341_WHITE);
  tft.fillRect(95, 197, 38, 21,  ILI9341_GREEN);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(100, 201);
  tft.setTextColor(ILI9341_BLACK);
  tft.print("RIT");
  ritEnable();
  ritOn = true;
//  Rit_Encoder = true;
//  PBT_Active = false;
  Display_RIT_Data();
 // Update_Mouse();
}


void Display_PBT_OFF(void) {

  tft.fillRect(IFx + 5, IFy - IFhi - 1 , IFwid - 1, IFhi + 3, ILI9341_BLUE);
  tft.fillRect(IFx + 14, IFy + 6 , 26, 10, ILI9341_BLACK);
  // draw PBT diagram

  tft.drawFastHLine(IFx, IFy, IFlip, ILI9341_WHITE);
  tft.drawFastVLine(IFx + IFlip - 1, IFy - IFhi - 2, IFhi + 2, ILI9341_WHITE);
  tft.drawFastHLine(IFx + IFlip - 1, IFy - IFhi - 2, IFwid, ILI9341_WHITE);
  tft.drawFastVLine(IFx + IFlip + IFwid - 1, IFy - IFhi - 2, IFhi + 2, ILI9341_WHITE);
  tft.drawFastHLine(IFx + IFlip + IFwid - 1, IFy, IFlip, ILI9341_WHITE);
  tft.setFont(Arial_14);
  tft.setTextColor(ILI9341_GREEN );
  tft.setCursor(IFx + 7, IFy - 22 );
  tft.println("PBT");

//  PBT_Active = false;
  PBT_ON = false;
  PBT_Value = 0;
  setFrequency(frequency);
//  Remember_Mouse();
 // Display_Mouse();

}




void Display_PBT_Position(void) {


  tft.fillRect(IFx + 5, IFy - IFhi - 1 , IFwid - 1, IFhi + 3, ILI9341_ORANGE);
  tft.setFont(Arial_14);
  tft.setTextColor(ILI9341_WHITE );
  tft.setCursor(IFx + 7, IFy - 22 );
  tft.println("L   R");

  if ( PBT_Value == 0 ) {
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 2, IFy - IFhi  , IFhi , ILI9341_BLACK);
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 3, IFy - IFhi  , IFhi , ILI9341_BLACK);
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 4, IFy - IFhi  , IFhi , ILI9341_BLACK);
  } else {
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 2 + (PBT_Value / 100)  , IFy - IFhi   , IFhi , ILI9341_RED);
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 3 + (PBT_Value / 100)  , IFy - IFhi   , IFhi , ILI9341_RED);
    tft.drawFastVLine(IFx + (IFwid / 2) + 1 + 4 + (PBT_Value / 100)  , IFy - IFhi   , IFhi , ILI9341_RED);

  }
  //PBT_Active = true;
  //Remember_Mouse();
// if ( (mouse_x > 20) && (mouse_x < 30) && (mouse_y > 120) && (mouse_y < 140))Remember_Mouse();

}

void Display_Version(void) {

  if ( Cat_Lock == 0 ) {
    tft.fillRect(164, 2, 25, 42,  ILI9341_BLACK);
    tft.drawRect(164, 2, 24, 28,  ILI9341_WHITE);
    tft.fillRect(165, 3, 22, 26,  ILI9341_BLUE);
    tft.setFont(Arial_12_Bold);
    tft.setTextColor(ILI9341_WHITE);
    tft.setCursor(167, 6);
    tft.print("V");
    tft.setCursor(177, 14);
    if ( Board_Version == 5 ) tft.print('5');
    else tft.print('L');
  }
}



void  Process_Board_Version( void ) {

  if ( Board_Version == 5 ) {
    Clock_Minimum = V5_CLOCK_MINIMUM;
    calibration = Get_SDEEPROM( V5_MASTER_CAL );
    usbCarrier  = Get_SDEEPROM( V5_USB_CAL );
    if ( (usbCarrier < 11000000) || (usbCarrier > 11100000)) {
      usbCarrier = 11059200;
      Put_SDEEPROM( V5_USB_CAL, usbCarrier );
      Pop_Up_Calibration_Notice();
    }
  } else {
    Clock_Minimum = LEGACY_CLOCK_MINIMUM;
    calibration = Get_SDEEPROM( MASTER_CAL );
    usbCarrier  = Get_SDEEPROM( USB_CAL );
    if ( (usbCarrier < 11800000) || (usbCarrier > 12100000)) {
      usbCarrier = 11997000;
      Put_SDEEPROM( USB_CAL, usbCarrier );
      Pop_Up_Calibration_Notice();
    }
  }
  Put_SDEEPROM( BOARD_VERSION, Board_Version );

  Display_Version();
}

  // draw smeter frame
void Display_Smeter(void){
  tft.setFont(Arial_16);
  tft.drawRect(94, 128, 208, 14, ILI9341_WHITE);
  tft.fillRect(95, 129, 206, 12, ILI9341_WHITE);
  tft.setFont(Arial_8_Bold);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(106, 118);
  tft.print("1    3      5        7          9");
  tft.setTextColor(ILI9341_RED);
  tft.setCursor(260, 118);
  tft.print("20      40");

}


void Init_Screen() {


  // display Memories
  Display_Mem_Selection();

  Display_Lock();
  if ( splitOn )Display_SPL_ON();
  else Display_SPL();
  Display_RIT();
  Display_Setup();

  //
  tft.drawRect(67, 47, 95, 21,  ILI9341_WHITE);
  tft.fillRect(68, 48, 93, 19,  ILI9341_BLUE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_12_Bold);
  tft.setCursor(93, 52 );
  tft.println("Band");

  Display_Trangle(70, 51, 16, 13, ILI9341_GREEN);
  Display_Trangle(140, 64, 16, -13, ILI9341_GREEN);


  Display_Smeter();

  // draw Clock
  if ( Clock_Active > 0 ) {
    //tft.drawRect(190, 2, 130, 28, ILI9341_WHITE);
    tft.fillRect(190, 2, 131, 28, ILI9341_WHITE);
  }

  Display_RX();

  tft.fillRect(306, 108, 11, 20, ILI9341_GREEN);
  tft.fillRect(306, 145, 11, 20, ILI9341_GREEN);
  //Display_Trangle(int xstart, int ystart, int width, int height, uint16_t color);
  Display_Trangle(303, 108, 16, -11, ILI9341_GREEN);
  Display_Trangle(303, 165, 16, 11, ILI9341_GREEN);


  Display_Version();

  Display_PBT_OFF();

}

void Fake_Delay1(void) {
  int fred = 2;
  while ( fred > 1 )fred = fred - 1;
}

void Fake_Delay(void) {
  long fred = 10;
  while ( fred > 1 ) {
    fred = fred - 1;
    delay(1);
  }
}


void Truncate_Frequency ( void ) {
  long rx_freq;

  if ( vfoActive == VFO_B ) rx_freq = vfoB;
  else rx_freq = vfoA;

  rx_freq = rx_freq / 1000;
  rx_freq = rx_freq * 1000.0;

  if ( vfoActive == VFO_B )vfoB = rx_freq;
  else vfoA = rx_freq;
  frequency = rx_freq;
  setFrequency(frequency);
  updateDisplay();


}




int Frequency_is_Band(unsigned long frequency) {
  int index = -1;
  char Response_Buffer[12];

  if ( frequency <=        4000000 )index = 0;
  else if ( frequency <=   7300000 )index = 1;
  else if ( frequency <=  10150000 )index = 2;
  else if ( frequency <=  14350000 )index = 3;
  else if ( frequency <=  18168000 )index = 4;
  else if ( frequency <=  21450000 )index = 5;
  else if ( frequency <=  24990000 )index = 6;
  else if (frequency <=   29700000 )index = 7;
  if ( Cat_Lock == true ) {
    sprintf(Response_Buffer, "uBX%1d;", Current_Band);
    Serial.write(Response_Buffer);
    delay(10);
  }
  return ( index );
}
////////////////////////////////////////////////////////////////////////////////////////////////////////


/**
   Below are the basic functions that control the uBitx. Understanding the functions before
   you start hacking around
*/
/**
   Select the properly TX harmonic filters
   The four harmonic filters use only three relays
   the four LPFs cover 30-21 MHz, 18 - 14 MHz, 7-10 MHz and 3.5 to 5 MHz
   Briefly, it works like this,
   - When KT1 is OFF, the 'off' position routes the PA output through the 30 MHz LPF
   - When KT1 is ON, it routes the PA output to KT2. Which is why you will see that
     the KT1 is on for the three other cases.
   - When the KT1 is ON and KT2 is off, the off position of KT2 routes the PA output
     to 18 MHz LPF (That also works for 14 MHz)
   - When KT1 is On, KT2 is On, it routes the PA output to KT3
   - KT3, when switched on selects the 7-10 MHz filter
   - KT3 when switched off selects the 3.5-5 MHz filter
   See the circuit to understand this
*/

void setTXFilters(unsigned long freq) {

  if ( Board_Version == 5 ) {

    if (freq > 21000000L) { // the default filter is with 35 MHz cut-off
      digitalWrite(TX_LPF_A, 0);
      digitalWrite(TX_LPF_B, 0);
      digitalWrite(TX_LPF_C, 0);
    }
    else if (freq >= 14000000L) { //thrown the KT1 relay on, the 30 MHz LPF is bypassed and the 14-18 MHz LPF is allowd to go through
      digitalWrite(TX_LPF_A, 1);
      digitalWrite(TX_LPF_B, 0);
      digitalWrite(TX_LPF_C, 0);
    }
    else if (freq > 7000000L) {
      digitalWrite(TX_LPF_A, 0);
      digitalWrite(TX_LPF_B, 1);
      digitalWrite(TX_LPF_C, 0);
    }
    else {
      digitalWrite(TX_LPF_A, 0);
      digitalWrite(TX_LPF_B, 0);
      digitalWrite(TX_LPF_C, 1);
    }
  } else {

    if (freq > 21000000L) { // the default filter is with 35 MHz cut-off
      digitalWrite(TX_LPF_A, 0);
      digitalWrite(TX_LPF_B, 0);
      digitalWrite(TX_LPF_C, 0);
    }
    else if (freq >= 14000000L) { //throw the KT1 relay on, the 30 MHz LPF is bypassed and the 14-18 MHz LPF is allowed to go through
      digitalWrite(TX_LPF_A, 1);
      digitalWrite(TX_LPF_B, 0);
      digitalWrite(TX_LPF_C, 0);
    }
    else if (freq > 7000000L) {
      digitalWrite(TX_LPF_A, 1);
      digitalWrite(TX_LPF_B, 1);
      digitalWrite(TX_LPF_C, 0);
    }
    else {
      digitalWrite(TX_LPF_A, 1);
      digitalWrite(TX_LPF_B, 1);
      digitalWrite(TX_LPF_C, 1);
    }
  }
}

void setFrequency(unsigned long xf) {
  unsigned long f;

  f = xf;
  frequency = xf;
  // CW filter if present should be used unless we are transmitting
  usbCarrier = Standard_Offset;  // assume not using CW filter
  if ( !inTx ) {
    // if (CW_Filter_Present) {
    //   if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR))usbCarrier = CW_Filter_Offset;
    //}
    usbCarrier = usbCarrier - PBT_Value;
    if ( ubitx_mode == MODE_CW) f = f - sideTone;
    if ( ubitx_mode == MODE_CWR) f = f + sideTone;
  }

  setTXFilters(f);

  si5351bx_setfreq(0, usbCarrier );

  if ( (ubitx_mode == MODE_SWU) || (ubitx_mode == MODE_USB) || (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWP)) {
    if ( Board_Version == 5 ) {
      si5351bx_setfreq(2, V5_SECOND_OSC_USB + f + ritOffset);
      si5351bx_setfreq(1, V5_SECOND_OSC_USB + usbCarrier);
    } else {
      si5351bx_setfreq(2, SECOND_OSC_USB - usbCarrier + f + ritOffset);
      si5351bx_setfreq(1, SECOND_OSC_USB);
    }
  } else {
    if ( Board_Version == 5 ) {
      si5351bx_setfreq(2, V5_SECOND_OSC_USB + f + ritOffset);
      si5351bx_setfreq(1, V5_SECOND_OSC_USB - usbCarrier);
    } else {
      si5351bx_setfreq(2, SECOND_OSC_LSB + usbCarrier + f + ritOffset);
      si5351bx_setfreq(1, SECOND_OSC_LSB);
    }
  }
  usbCarrier = Standard_Offset;  // assume not using CW filter

  Last_Frequency_Save_Ctr = 4;
}

/**
   startTx is called by the PTT, cw keyer and CAT protocol to
   put the uBitx in TX mode. It takes care of RIT settings, sideband settings
   Note: In cw mode, doesn't key the radio, only puts it in TX mode
*/

void startTx(byte txMode) {
  unsigned long tx_freq;

  if ( ubitx_mode != MODE_CWP) {
    if ( splitOn ) tx_freq = vfoB;
    else tx_freq = frequency;
    setFrequency(tx_freq);
    if (txMode == TX_CW) {
      //turn off the second local oscillator and the BFO
      si5351bx_setfreq(0, 0);
      si5351bx_setfreq(1, 0);
      si5351bx_setfreq(2, tx_freq );
    }

    Display_TX();
    inTx = true;
    digitalWrite(TX_RX, 1);
  }
}
void stopTx(void) {

  digitalWrite(TX_RX, 0);           //turn off the TX
  inTx = false;
  Display_RX();
  si5351bx_setfreq(0, usbCarrier);  //set back the carrier oscillator anyway, CW TX switches it off
  if ( splitOn )setFrequency(vfoA);
  setFrequency(frequency);
}

/**
   ritEnable is called with a frequency parameter that determines
   what the RX frequency will be
*/
void ritEnable(void) {
  if (!ritOn) {
    ritOffset = LastritOffset;
    setFrequency(frequency);
    updateDisplay();
    Display_Mem_Selection();
  }
}

// this is called by the RIT menu routine
void ritDisable(void) {
  if (ritOn) {
    LastritOffset = ritOffset;
    ritOffset = 0;
    setFrequency(frequency);
    updateDisplay();
    Display_Mem_Selection();
  }
}


/**
   Basic User Interface Routines. These check the front panel for any activity
*/


void Save_Radio_State( void ) {

  if (vfoActive == VFO_A) {
    Put_SDEEPROM( VFO_A, frequency );
  } else {
    Put_SDEEPROM( VFO_B, frequency );
  }

}

bool Get_Button( int button ) {

  if (digitalRead(button) == HIGH) return false;
  else return true;

}

// test button for short or long push
int Test_Button( int button, char * title1, char * title2, char * title3, char * title4 ) {
  int i;

  //only if the button is pressed
  if (!Get_Button(button)) return 0;
  delay(100);
  if (!Get_Button(button)) return 0; //debounce
  // OK button is definitely pushed so now detect how long it is pressed.

  if ( Cat_Lock == true ) {
    while ( Get_Button(button)) delay(10);
    return -1;
  } else {
    tft.setFont(Arial_14_Bold);
    tft.setTextColor(ILI9341_BLUE);

    i = 0;
    while ( Get_Button(button) ) {
      delay(10);
      i++;
      if ( (i >= BUTTON_DELAY_1) && (i < BUTTON_DELAY_2) ) {
        if ( i == BUTTON_DELAY_1 ) {
          tft.fillRect(2, 192, 120, 14, ILI9341_WHITE);
          tft.setCursor(2, 192);
          tft.print(title1);
        }
      }
      if ( (i >= BUTTON_DELAY_2) && (i < BUTTON_DELAY_3) ) {
        if ( i == BUTTON_DELAY_2 ) {
          tft.fillRect(2, 192, 120, 14, ILI9341_WHITE);
          tft.setCursor(2, 192);
          tft.print(title2);
        }
      }
      if ((i >= BUTTON_DELAY_3) && (i < BUTTON_DELAY_4) ) {
        if ( i == BUTTON_DELAY_3 ) {
          tft.fillRect(2, 192, 120, 14, ILI9341_WHITE);
          tft.setCursor(2, 192);
          tft.print(title3);
        }
      }
      if ( i >= BUTTON_DELAY_4) {
        if ( i == BUTTON_DELAY_4 ) {
          tft.fillRect(2, 192, 120, 14, ILI9341_WHITE);
          tft.setCursor(2, 192);
          tft.print(title4);
        }
      }
    }
    tft.fillRect(2, 192, 120, 14, ILI9341_WHITE);

    if ((i > BUTTON_DELAY_1) && (i < BUTTON_DELAY_2) )return 1;
    if ((i > BUTTON_DELAY_2 ) && (i < BUTTON_DELAY_3) )return 2;
    if ((i > BUTTON_DELAY_3 ) && (i < BUTTON_DELAY_4) )return 3;
    if ( i > BUTTON_DELAY_4) return 4;
    else return -1;
  }
}

int Center_Freq = 40;
int current_signal = 0;
int signal_diff = 0;
int last_diff = 0;

void doTuning() {
  int s = 0;
  long rx_freq;


//  if ( !PBT_Active ) {
    s = enc_read();
    if ( !Tune_Lock ) {
      if (s) {
        if ( vfoActive == VFO_B ) rx_freq = vfoB;
        else rx_freq = vfoA;

        if (s > 4)
          rx_freq += Freq_Incr[Freq_Index];
        else if (s >  2)
          rx_freq += Freq_Incr[Freq_Index];
        else if (s >  0)
          rx_freq +=  Freq_Incr[Freq_Index];
        else if (s <  0)
          rx_freq -= Freq_Incr[Freq_Index];
        else if (s < -2)
          rx_freq -= Freq_Incr[Freq_Index];
        else if (s < -4)
          rx_freq -= Freq_Incr[Freq_Index];

        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;

        if ( vfoActive == VFO_B )vfoB = rx_freq;
        else vfoA = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
      }
    }
//  }
}
/*
  void Do_CW_CAL( void ) {
  int s, i;
  int Local_Freq_Index = 2;


  printLine1((char*)"Choose - Exit");
  printLine2((char*)"tune until ");
  printLine3((char*)"sounds good ");
  printLine4((char*)"  ");
  CW_Filter_Offset = Get_SDEEPROM( CW_FILTER_OFFSET );
  CW_Filter_Offset = 8000000l;
  si5351bx_setfreq(0, CW_Filter_Offset);
  setFrequency(frequency);
  // while (!Choose_Down()) {
  s = enc_read();
  if (s) {
    if (s > 4)
      CW_Filter_Offset += Freq_Incr[Local_Freq_Index];
    else if (s >  2)
      CW_Filter_Offset += Freq_Incr[Local_Freq_Index];
    else if (s >  0)
      CW_Filter_Offset +=  Freq_Incr[Local_Freq_Index];
    else if (s <  0)
      CW_Filter_Offset -= Freq_Incr[Local_Freq_Index];
    else if (s < -2)
      CW_Filter_Offset -= Freq_Incr[Local_Freq_Index];
    else if (s < -4)
      CW_Filter_Offset -= Freq_Incr[Local_Freq_Index];

    si5351bx_setfreq(0, CW_Filter_Offset);
    setFrequency(frequency);
  }
  // check freq incr
  if (digitalRead(FBUTTON) == LOW) {
    i = 0;
    while ( digitalRead(FBUTTON) == LOW ) {
      delay(10);
      i++;
    }
    if ( i > 1) {
      Local_Freq_Index = Local_Freq_Index - 1;
      if ( Local_Freq_Index < 0 )Local_Freq_Index = 5;
    }
  }
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));
  ultoa(CW_Filter_Offset, b, DEC);
  if (strlen(b) < 7) {
    strcat(&c[0], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);
  } else if (strlen(b) < 8) {
    strcat(&c[0], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 2);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);
  }

  //printLineXI(3, 11, c);
  //}
  // while (Choose_Down())delay(10);

  Put_SDEEPROM( CW_FILTER_OFFSET, CW_Filter_Offset );
  Put_SDEEPROM( CW_FILTER_CALIBRATED, true );
  CW_Calibrated = true;

  updateDisplay();

  }
*/

/**
   RIT only steps back and forth by 100 Hz at a time
*/
void doRIT() {
  int knob = 0;


//  if ( Rit_Encoder ) {
    knob = enc_read();


    if (knob != 0 ) {
      if (knob < 0) {
        ritOffset -= 100l;
        if ( abs(ritOffset) > ritOffsetMax) ritOffset = -ritOffsetMax;
      } else if (knob > 0) {
        ritOffset += 100;
        if ( ritOffset > ritOffsetMax) ritOffset = ritOffsetMax;
      }
      setFrequency(frequency);
      updateDisplay();
    }
//  }
}

/**
   The settings are read from EEPROM. The first time around, the values may not be
   present or out of range, in this case, some intelligent defaults are copied into the
   variables.
*/
void initSettings() {
  int i;

  //  Reading Board Version MUST BE FIRST

  // process uBITX board version support
  Board_Version = Get_SDEEPROM( BOARD_VERSION );
  if ( (Board_Version < 1) || (Board_Version > 5)) {
    Board_Version = 5;
  }
  Process_Board_Version();


  Standard_Offset = usbCarrier;


  SI5351BX_XTALPF = Get_SDEEPROM( XTALPF );
  if ( (SI5351BX_XTALPF < 1) || (SI5351BX_XTALPF > 3)) {
    SI5351BX_XTALPF = 2;
    Put_SDEEPROM( XTALPF, SI5351BX_XTALPF );
  }

  sideTone = Get_SDEEPROM( CW_SIDETONE );
  if ( sideTone < 300 || sideTone > 2000 ) sideTone = 800;

  wpm = Get_SDEEPROM( IAMBIC_MODE );
  IAMBIC = (volatile unsigned char)wpm;
  if ( IAMBIC == 0 )SetIambicA();
  else SetIambicB();

  wpm = Get_SDEEPROM( PDL_SWAP_VAL );
  PDLSWAP = (volatile unsigned char)wpm;
  if ( PDLSWAP == 8 ) SetPaddleSwapR();
  else SetPaddleSwapN();

  wpm = Get_SDEEPROM( MODE );
  if ( wpm < 0 ) wpm = 0;
  if ( wpm > 6 ) wpm = 6;
  ubitx_mode = wpm;
  Last_Mode = wpm;
  if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR) || (ubitx_mode == MODE_CWP)) CW_Active = true;
  else CW_Active = false;


  wpm = Get_SDEEPROM( VOLTS );
  if ( wpm == 1 ) {
    Volts_Active = true;
  } else {
    Volts_Active = false;
  }

  ubitx_baud = Get_SDEEPROM( BAUDD );
  if ( ubitx_baud < 0 ) ubitx_baud = 5;
  if ( ubitx_baud > 7 ) ubitx_baud = 5;
  Last_Baud = ubitx_baud;

  wpm = Get_SDEEPROM( CW_SPEED );
  if ( wpm < 5 ) wpm = 5;
  if ( wpm > 50 ) wpm = 50;
  cwSpeed = 600 / wpm;

  Freq_Index = Get_SDEEPROM( FREQ_INCR );
  if ( (Freq_Index < 0) || (Freq_Index > 5)) {
    Freq_Index = 2;
    Put_SDEEPROM( FREQ_INCR, Freq_Index );
    delay(200);
  }



  Smeter_Active = Get_SDEEPROM( SMETER_ACTIVE );
  Clock_Active = Get_SDEEPROM( CLOCK_ACTIVE );


  Volt_Divider = Get_SDEEPROM( VOLT_DIVIDE );
  if ( (Volt_Divider > 2000) || (Volt_Divider <= 0 )) Volt_Divider = 982;


  Memory_Table_Select = Get_SDEEPROM( SELECTED_MEM_TABLE );
  if ( (Memory_Table_Select > 3) || (Memory_Table_Select < 0))Memory_Table_Select = 0;
  for (i = 0; i < 10; i++) {
    Working_Copy_Frequencies[i] = Get_SDEEPROM( ((Memory_Table_Select + 1) * 100 ) + (i * 4) );
    Working_Copy_Modes[i] = Get_SDEEPROM( ((Memory_Table_Select + 1) * 100 ) + 40 + (i * 4) );
  }

  //CW_Filter_Present = Get_SDEEPROM( CW_FILTER_PRESENT );
  //CW_Calibrated = Get_SDEEPROM( CW_FILTER_CALIBRATED );
  //CW_Filter_Offset = Get_SDEEPROM( CW_FILTER_OFFSET );
  //if ( !CW_Filter_Present )CW_Filter_Offset = usbCarrier;


  // try to open screen calibration file
  fileptr = SD.open("SCAL", FILE_READ);
  if ( fileptr ) {
    Read_Screen_CAL_to_SD();
    fileptr.close();
  } else {
    CAL_Minx = 5;
    CAL_Maxx =  330;
    CAL_Miny = 5;
    CAL_Maxy = 220;

    Calibrate_Screen();
  }

  // try to open Band Table file
  fileptr = SD.open("BTBL", FILE_READ);
  if ( fileptr == 0 ) {
    Write_Entire_Band_Table_to_SD();
  } fileptr.close();

  vfoB = Get_SDEEPROM( VFO_B );
  if ( vfoB <   400000l ) vfoB = 400000l;
  if ( vfoB > 30000000l ) vfoB = 30000000l;
  VFOB_BAND = Frequency_is_Band(vfoB);

  vfoA = Get_SDEEPROM( VFO_A );
  if ( vfoA <   400000l ) vfoA = 400000l;
  if ( vfoA > 30000000l ) vfoA = 30000000l;
  VFOA_BAND = Frequency_is_Band(vfoA);
  frequency = vfoA;
  setFrequency(frequency);
  vfoActive = VFO_A;


}




void initPorts() {


  pinMode(ENC_A, INPUT_PULLUP);
  pinMode(ENC_B, INPUT_PULLUP);
  pinMode(FBUTTON, INPUT_PULLUP);
  pinMode(UBIT_VCC, INPUT );
  pinMode(SMETERREADPIN, INPUT );

  // init CW digital pins
  pinMode(DIGITAL_DOT, INPUT_PULLUP);
  pinMode(DIGITAL_DASH, INPUT_PULLUP);
  pinMode(DIGITAL_KEY, INPUT_PULLUP);

  pinMode(CW_TONE, OUTPUT);
  digitalWrite(CW_TONE, 0);

  pinMode(TX_RX, OUTPUT);
  digitalWrite(TX_RX, 0);

  pinMode(TX_LPF_A, OUTPUT);
  pinMode(TX_LPF_B, OUTPUT);
  pinMode(TX_LPF_C, OUTPUT);
  digitalWrite(TX_LPF_A, 0);
  digitalWrite(TX_LPF_B, 0);
  digitalWrite(TX_LPF_C, 0);

  pinMode(CW_KEY, OUTPUT);
  digitalWrite(CW_KEY, 0);

}


void Set_Band( void) {

  Read_Band_Table_from_SD();
  Init_Screen();
  LastSmeter = 0;
  Display_S_Meter();
  Display_Mode();
  setFrequency(frequency);
  updateDisplay();
}

///////////////////////////////////////////////////////

int Last_Second = 0;
int Last_Minute = 0;

void digitalClockDisplay() {
  char Time_Buffer[21];
  uint8_t oldSREG;
  int index;


  // digital clock display of the time
  if ( Last_Second != second()) {
    Last_Second = second();
    if ( Set_Time ) {
      Set_Time = false;
      setTime(pctime);
      Teensy3Clock.set(now());
    }
    if ( Clock_Active ) {
      if (  Keyboard_Mode != KEY_CW ) {
        strcpy(Time_Buffer, "");
        sprintf(Time_Buffer, "%02d:%02d:%02d", hour(), minute(), second());
        tft.setTextColor(ILI9341_BLUE);
        tft.setFont(Arial_20_Bold);
        tft.fillRect(190, 4, 124, 26, ILI9341_WHITE);
        tft.setCursor(196, 6);
        tft.print(Time_Buffer);
        strcpy(Time_Buffer, "");
      }
    }

    for (uint8_t i = 0; i < CNT_DEVICES; i++) {
      if (*drivers[i] != driver_active[i]) {
        if (driver_active[i]) {
#ifdef DEBUG
          Serial.printf("*** Device %s - disconnected ***\n", driver_names[i]);
#endif
          driver_active[i] = false;
        } else {
#ifdef DEBUG
          Serial.printf("*** Device %s %x:%x - connected ***\n", driver_names[i], drivers[i]->idVendor(), drivers[i]->idProduct());
#endif
          driver_active[i] = true;

#ifdef DEBUG
          const uint8_t *psz = drivers[i]->manufacturer();
          if (psz && *psz) Serial.printf("  manufacturer: %s\n", psz);
          psz = drivers[i]->product();
          if (psz && *psz) Serial.printf("  product: %s\n", psz);
          psz = drivers[i]->serialNumber();
          if (psz && *psz) Serial.printf("  Serial: %s\n", psz);
#endif
        }
      }
    }

    for (uint8_t i = 0; i < CNT_HIDDEVICES; i++) {
      if (*hiddrivers[i] != hid_driver_active[i]) {
        if (hid_driver_active[i]) {
#ifdef DEBUG
          Serial.printf("*** HID Device %s - disconnected ***\n", hid_driver_names[i]);
#endif
          hid_driver_active[i] = false;
          Remove_Mouse();
          Mouse_Active = false;
        } else {
#ifdef DEBUG
          Serial.printf("*** HID Device %s %x:%x - connected ***\n", hid_driver_names[i], hiddrivers[i]->idVendor(), hiddrivers[i]->idProduct());
#endif
          hid_driver_active[i] = true;
          Mouse_Active = true;
          Display_Mouse();

#ifdef DEBUG
          const uint8_t *psz = hiddrivers[i]->manufacturer();
          if (psz && *psz) Serial.printf("  manufacturer: %s\n", psz);
          psz = hiddrivers[i]->product();
          if (psz && *psz) Serial.printf("  product: %s\n", psz);
          psz = hiddrivers[i]->serialNumber();
          if (psz && *psz) Serial.printf("  Serial: %s\n", psz);
#endif
        }
      }
    }

    // process VFO save

    if ( Last_Frequency_Save_Ctr > 0 ) {
      Last_Frequency_Save_Ctr--;

      if ( Last_Frequency_Save_Ctr <= 0 ) {
        Last_Frequency_Save_Ctr = 0;
        if ( Last_New_Frequency != frequency ) {
          if ( vfoActive == VFO_B ) {
            Put_SDEEPROM( VFO_B, frequency );
          } else {
            Put_SDEEPROM( VFO_A, frequency );
          }
          Last_New_Frequency = frequency;
          index = -1;
          if ( frequency <=        4000000 )index = 0;
          else if ( frequency <=   7300000 )index = 1;
          else if ( frequency <=  10150000 )index = 2;
          else if ( frequency <=  14350000 )index = 3;
          else if ( frequency <=  18168000 )index = 4;
          else if ( frequency <=  21450000 )index = 5;
          else if ( frequency <=  24990000 )index = 6;
          else if (frequency <=   29700000 )index = 7;
          if ( index != -1) {
            Current_Band = index;
            Write_Band_Table_Update_to_SD();
          }
        }
      }
    }
    if ( Volts_Delay_Ctr <= 0) {
      Volts_Delay_Ctr = 5;
      if ( Volts_Active ) {


        oldSREG = SREG;
        cli();
        Ubitx_Voltage = analogRead(UBIT_VCC);
        Volts_Active = true;
        Ubitx_Voltage = analogRead(UBIT_VCC);
        SREG = oldSREG;


        if ( Volts_Active != Current_Ubitx_Voltage ) {
          Current_Ubitx_Voltage = Volts_Active;
          Display_Voltage();
        }
      }


    } else {
      Volts_Delay_Ctr--;
    }
    Place_The_Cursor();

  }
}


time_t getTeensy3Time()
{
  return Teensy3Clock.get();
}

void uBITX_Keyboard_Press(int key) {

  Keyboard_Input_Buffer[0] = uBITX_Keyboard.getKey();
  KeyIndex = 1;

}



void Process_Setup( void ) {
  bool looper = true;

  tft.fillScreen(ILI9341_BLUE);
  tft.drawRect(138, 100, 172, 119,  ILI9341_WHITE);
  tft.fillRect(139, 101, 170, 117, ILI9341_BLUE);
  tft.setFont(Arial_16);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(141, 108);
  tft.println((char*)"CW Options");
  tft.setCursor(141, 138);
  tft.println((char*)"Configuration 2");
  tft.setCursor(141, 168);
  tft.println((char*)"Configuration 1");
  tft.setCursor(141, 198);
  tft.println((char*)"Calibrate Radio");
  Update_Mouse();

  while ( looper ) {
    if (ts.touched()) {
      Get_Touch(true);
      if ( (p.x > 140) && (p.x < 300) && (p.y > 198) && (p.y < 230)) Do_Calibrate_Radio();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 168) && (p.y < 188)) Do_Configuration1();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 138) && (p.y < 158)) Do_Configuration2();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 108) && (p.y < 128)) Do_Mouse_CW_Options();
      looper = false;
    }
    if (Set_Left_Button_Pressed() == 1) {
      while ( Set_Left_Button_Pressed() == 1) Fake_Delay1();
      if ( (p.x > 140) && (p.x < 300) && (p.y > 198) && (p.y < 218)) Do_Calibrate_Radio();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 168) && (p.y < 188)) Do_Configuration1();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 138) && (p.y < 158)) Do_Configuration2();
      else if ( (p.x > 140) && (p.x < 300) && (p.y > 108) && (p.y < 128)) Do_Mouse_CW_Options();
      looper = false;
    }
    Fake_Delay();
  }
  Process_Reboot();
  Update_Mouse();

}



//////////////////////////////////////////////////////////////////////////////////////////

void Process_Reboot(void) {

  tft.fillScreen(ILI9341_BLACK);
  LastSmeter = 0;
  initSettings();
  initPorts();
  initOscillators();
  Init_Screen();
  menuVfoToggle();
  menuVfoToggle();
  Display_Mode();
  keyerState = IDLE;

}

void setup() {

  Wire2.begin(I2C_MASTER, 0x00, I2C_PINS_3_4, I2C_PULLUP_EXT, 400000);

  tft.begin();
  ts.begin();
  tft.setRotation(0);
  tft.fillScreen(GLCD_CL_SILVER);
  tft.setRotation(3);
  //we print this line so this shows up even if the Raduino
  //crashes later in the code
  //tftPrint(int x,int y,char* Text,ILI9341_t3_font_t,int fontColor,bool textframed,int frameColor );
  tftPrint(30, 10, (char*)"T", Arial_32_Bold, ILI9341_RED, 0, ILI9341_RED);
  tftPrint(60, 10, (char*)"riumvirate", Arial_32_Bold, ILI9341_GREEN, 0, ILI9341_GREEN);

  tftPrint(60, 50, (char*)"S", Arial_32_Bold, ILI9341_RED, 0, ILI9341_RED);
  tftPrint(100, 50, (char*)"konk", Arial_32_Bold, ILI9341_GREEN, 0, ILI9341_GREEN);
  tftPrint(90, 90, (char*)"W", Arial_32_Bold, ILI9341_RED, 0, ILI9341_RED);
  tftPrint(140, 90, (char*)"orx", Arial_32_Bold, ILI9341_GREEN, 0, ILI9341_GREEN);

  tft.setFont(Arial_16_Bold);
  tft.setTextColor(ILI9341_DARKGREEN);
  tft.setCursor(60, 130);
  tft.print("W0EB N5IB W2CTX");
  tft.setFont(Arial_18_Bold);
  tft.setTextColor(ILI9341_RED);
  tft.setCursor(70, 150);
  tft.print("www.w0eb.com");

  tft.setFont(Arial_24_Bold);
  tft.setTextColor(ILI9341_BLUE);
  tft.setCursor(4, 174);
  tft.print(Version_Text);

  tft.setFont(Arial_24_Bold);
  tft.setCursor(26, 210);
  tft.setTextColor(ILI9341_BLUE);
  tft.print(Version_Date);

  delay(3000);
  SD_Setup();
  // set the Time library to use Teensy 3.0's RTC to keep time
  setSyncProvider(getTeensy3Time);
  Teensy3Clock.get();
  uBITX_USB.begin();
  uBITX_Keyboard.attachPress(uBITX_Keyboard_Press);

  Process_Reboot();

  if (digitalRead(FBUTTON) == LOW) {
    delay(200);
    if (digitalRead(FBUTTON) == LOW) {
      CAL_Minx = 305;
      CAL_Maxx =  3400;
      CAL_Miny = 305;
      CAL_Maxy = 3500;
      Calibrate_Screen();
      Process_Reboot();
    }
  }

  Init_Cat( ubitx_CAT_Long_Baud_rates[ubitx_baud] );

  Connect_Interrupts();


}





void checkFreqIncr_RIT( void ) {
  int i, j;

  if (digitalRead(FBUTTON) == LOW) {
    delay(100);
    if (digitalRead(FBUTTON) == HIGH) return; //debounce
    i = 0;
    while ( digitalRead(FBUTTON) == LOW) {
      delay(10);
      i++;
      if ( i > BUTTON_DELAY_1 ) {
        j = Freq_Index;
        Freq_Index = Freq_Index + 1;
        if ( Freq_Index > 5 )Freq_Index = 0;
        Place_The_Cursor();
        Freq_Index = j;
      }
    }
    if ( i > BUTTON_DELAY_1 ) {
      if ( !ritOn) {
        delay(100);
        Freq_Index = Freq_Index + 1;
        if ( Freq_Index > 5 )Freq_Index = 0;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        delay(200);
        updateDisplay();
      }
    } else {
      if ( !ritOn) {
        delay(100);
        Freq_Index = Freq_Index - 1;
        if ( Freq_Index < 0 )Freq_Index = 5;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        delay(200);
        updateDisplay();
      }
    }
    if ( Freq_Index > 1 )Truncate_Frequency();
  }

}







void Process_Memory_Select( void) {


  Mem_Number = Mem_Number + 1;
  if ( Mem_Number > 9 ) Mem_Number = 0;
  Display_Mem_Selection();
}

void Process_Memory_Load( void) {

  ubitx_mode = Working_Copy_Modes[Mem_Number];
  if (vfoActive == VFO_B)vfoB = Working_Copy_Frequencies[Mem_Number];
  else vfoA = Working_Copy_Frequencies[Mem_Number];
  frequency = Working_Copy_Frequencies[Mem_Number];
  Put_SDEEPROM( MODE, ubitx_mode );
  delay(200);
  setFrequency(frequency);

  tft.setFont(Arial_12_Bold);
  tft.setTextColor(ILI9341_ORANGE);
  tft.fillRect(88, 5, 24, 16, ILI9341_BLACK );
  tft.setCursor(90, 6);
  tft.println("Ld");

  delay(1000);

  Display_Mem_Selection();
  Display_Mode();
  updateDisplay();

}



extern char Memory_Tables[4][4];

void Process_Memory_Tables( void) {
  int Mem_Table;
  int i;
  int Mem_Table_Ptr;
  ///////////////////////////

  ///////////////////////////

  Mem_Table = Memory_Table_Select;
  //  Mem_Table = Mem_Table + 1;
  if ( Mem_Table > 3 ) Mem_Table = 0;
  Memory_Table_Select = Mem_Table;
  Mem_Table_Ptr = ( Mem_Table + 1 ) * 100;
  for (i = 0; i < 10; i++) {
    Working_Copy_Frequencies[i] = Get_SDEEPROM( Mem_Table_Ptr + (i * 4) );
    Working_Copy_Modes[i] = Get_SDEEPROM( Mem_Table_Ptr + 40 + (i * 4) );
  }
  Put_SDEEPROM( SELECTED_MEM_TABLE, Mem_Table );

  Display_Mem_Selection();
  updateDisplay();


}

////////////////////////////////////////////////////////   L  O  O  P  /////////////////////////
void loop() {



  Process_CAT_Command();

  if ( KeyIndex == 1 ) Process_Keyboard();

  //do tasks only when not tranmsitting
  if (!inTx) {
    digitalClockDisplay()  ;
    if ( Smeter_Active == true ) {

      if ( smeterInterval <= 0 ) {
        smeterInterval = 150;
        //smeterInterval = 500;
        Display_S_Meter();
      }
    }

    if ( Cat_Lock == false ) {
      Process_Mouse();
      Process_Buttons();
      Process_Mouse();
      checkFreqIncr_RIT();
      if (ritOn) doRIT();
      else doTuning();
      Process_Mouse();
    } else {
      doTuning();
      checkFreqIncr_RIT();
    }
  }
}
