/** Menus
    File Name ubitx_menu.cpp
*/

#include <Arduino.h>
#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <SPIN.h>
#include <XPT2046_Touchscreen.h>
#include <SPI.h>
#include <SD.h>
#include <EEPROM.h>
#include "ubitx.h"

extern ILI9341_t3n tft;
extern XPT2046_Touchscreen ts;
extern TS_Point p;

extern void Display_Current_Speed(bool display_speed);

extern void tftPrint(int x,
                     int y,
                     char* Text,
                     ILI9341_t3_font_t font,
                     int fontColor,
                     bool textframed,
                     int frameColor );

extern bool Set_Left_Button_Pressed(void);
extern bool Get_Left_Button_Pressed(void);
extern int Get_Mouse_Wheel_Value(void);
extern void Update_Mouse(void);
extern void Save_Mouse(void);
extern void Fake_Delay(void);

extern void Display_SPL(void);
extern void Display_SPL_ON(void);
extern void Display_Lock(void);
extern void Display_Lock_ON(void);
extern void Display_RIT_ON(void);
extern void Display_RIT(void);
extern int  enc_read(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );
extern void sendKey(char c);
extern void Get_Touch( bool wait );
extern void Set_Band(void);
extern int Frequency_is_Band(unsigned long frequency);

extern volatile boolean Key_Send_Buffer_Full;
extern volatile int     Key_Send_Buffer_IN_Index;
extern volatile int     Key_Send_Buffer_OUT_Index;
extern volatile int     Key_Send_Buffer_Count;
extern int VFOA_BAND;
extern int VFOB_BAND;

extern char Fill_Text[12];
extern char Last_VFO_A_Text[12];
extern char Last_VFO_B_Text[12];


extern void Display_Mode();

extern unsigned long frequency;
extern char ritOn;
extern char splitOn;
extern char vfoActive;
extern unsigned long vfoA;
extern unsigned long vfoB;
extern int cwSpeed;
extern char c[21], b[21];
extern unsigned long cwTimeout;
extern unsigned long sideTone;
extern int Menu_Index;                      // new menu
extern volatile int ubitx_mode;
extern unsigned volatile char IAMBIC;  // 0 for Iambic A, 0x10 for Iambic B
extern unsigned volatile char PDLSWAP;     // 0x08 for normal, 0x00 for swap
extern volatile unsigned char keyerControl;
extern bool Volts_Active;
extern bool Tune_Lock;
extern int Current_Band;

extern char Keyboard_Input_Buffer[40];
int KeyIndex = 0;
int OutKeyIndex = 0;
int SendKeyIndex = 0;
int Keyboard_Value;

bool Key_Displayed = false;
bool Creating_Buffered_Text = false;
int Keyboard_Mode = 0;
bool Yellow_Key_Displayed = false;
bool Red_Key_Displayed = false;

//extern int Memory_Number;
bool Memories_Active = false;

extern bool VFO_A_Displayed;
extern bool VFO_B_Displayed;

int wpm;
extern int Mem_Number;
int Memory_Table_Select = 0;

int Menu_Cursor_Line_Postions[9] =  {1, 1, 1, 1, 1};
int Menu_Cursor_Index_Postions[9] = {0, 5, 6, 8, 13};
int Max_Menus_Items = 4;
int Current_Menu = 0;
int Last_Menu = 0;
int Last_Mode = 0;

////////////////////////////////////////////////////////////



char Key_buffer0[2];
char Key_buffer1[MAXKYBDENTRY + 2];
char Key_buffer2[MAXKYBDENTRY + 2];
unsigned long Key_Freq;

////////////////////////////////////////////////////////////




void SetIambicA( void ) {
  int temp;

  IAMBIC = 0;
  keyerControl = keyerControl & 0xEF;
  temp = IAMBIC;
  Put_SDEEPROM( IAMBIC_MODE, temp );
}

void SetIambicB( void ) {
  int temp;

  IAMBIC = 0x10;
  keyerControl = keyerControl | IAMBIC;
  temp = IAMBIC;
  Put_SDEEPROM( IAMBIC_MODE, temp );
}

void menuIambicToggle( void ) {

  if ( IAMBIC == 0 )SetIambicB();
  else SetIambicA();

}

void SetPaddleSwapN(void ) {

  PDLSWAP = 0;
  keyerControl = keyerControl & 0xF7;
  Put_SDEEPROM( PDL_SWAP_VAL, PDLSWAP );

}

void SetPaddleSwapR( void ) {

  PDLSWAP = 0x08;
  keyerControl = keyerControl | PDLSWAP;
  Put_SDEEPROM( PDL_SWAP_VAL, PDLSWAP );

}



void menuPaddleSwapToggle( void ) {

  if ( PDLSWAP == 0 ) SetPaddleSwapR();
  else  SetPaddleSwapN();


}


void menuVfoToggle( void ) {

  if (splitOn) {
    if (vfoActive == VFO_B) {
      vfoB = frequency;
      VFOA_BAND = Frequency_is_Band(vfoB);
      Put_SDEEPROM( VFO_B, frequency );

      vfoActive = VFO_A;
      frequency = vfoA;
      VFOA_BAND = Frequency_is_Band(vfoA);
      Current_Band = VFOA_BAND;
      VFO_B_Displayed = false;
    } else {
      vfoA = frequency;
      VFOA_BAND = Frequency_is_Band(vfoA);
      Put_SDEEPROM( VFO_A, frequency );

      vfoActive = VFO_B;
      frequency = vfoB;
      VFOB_BAND = Frequency_is_Band(vfoB);
      Current_Band = VFOB_BAND;
      VFO_A_Displayed = false;
    }
  } else {
    if (vfoActive == VFO_B) {
      vfoB = frequency;
      VFOA_BAND = Frequency_is_Band(vfoB);
      Put_SDEEPROM( VFO_B, frequency );

      vfoActive = VFO_A;
      frequency = vfoA;
      VFOA_BAND = Frequency_is_Band(vfoA);
      Current_Band = VFOA_BAND;
      VFO_B_Displayed = false;

    } else {
      vfoA = frequency;
      VFOA_BAND = Frequency_is_Band(vfoA);
      Put_SDEEPROM( VFO_A, frequency );

      vfoActive = VFO_B;
      frequency = vfoB;
      VFOB_BAND = Frequency_is_Band(vfoB);
      Current_Band = VFOB_BAND;
      VFO_A_Displayed = false;
    }
  }

  if ( vfoActive == VFO_A ) {
    tft.setTextColor(ILI9341_LIGHTGREY);
  } else {
    tft.setTextColor(ILI9341_ORANGE);
    tft.fillRect(64, 76, 40, 42, ILI9341_BLACK);
  }
  if (splitOn)strcpy(b, "T ");
  else strcpy(b, "B ");
  tft.setFont(Arial_32_Bold);
  tft.setCursor(64, 76);
  tft.println(b);


  if ( vfoActive == VFO_A ) {
    tft.setTextColor(ILI9341_ORANGE);
    tft.fillRect(64, 144, 40, 42, ILI9341_BLACK);
  } else {
    tft.setTextColor(ILI9341_LIGHTGREY);
  }
  if (splitOn) strcpy(b, "R " );
  else strcpy(b, "A ");
  tft.setFont(Arial_32_Bold);
  tft.setCursor(64, 154);
  tft.println(b);

  strcpy(Last_VFO_A_Text, Fill_Text);
  strcpy(Last_VFO_B_Text, Fill_Text);
  setFrequency(frequency);
  updateDisplay();

}

void menuRitToggle( void ) {
  if (ritOn == 0) {
    Display_RIT_ON();
  } else {
    Display_RIT();
  }
  delay(100);
  updateDisplay();
}


void menuSplitToggle( void ) {

  if ( splitOn ) {
    Display_SPL();
  } else {
    Display_RIT();
    Display_SPL_ON();
    vfoB = vfoA;
  }
  menuVfoToggle();
  updateDisplay();
  menuVfoToggle();
  updateDisplay();
}




void menuCWSpeed( void ) {
  int knob = 0;
  bool looper = true;

  tft.setFont(Arial_18);
  tft.setTextColor(ILI9341_ORANGE);
  tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
  tft.setCursor(24, 48 );
  itoa(wpm, b, 10);
  tft.println(b);
  Update_Mouse();
  while (ts.touched()) delay(10);
  while (Get_Left_Button_Pressed() == 1)delay(10);
  while ( looper ) {
    if ( ts.touched() ) {
      Get_Touch(true);
      if ( (p.x > 6) && (p.x < 75) && (p.y > 40) && (p.y < 55)) looper = false;
    }
    if ((Set_Left_Button_Pressed() == 1) && (p.x > 24) && (p.x < 42) && (p.y > 45) && (p.y < 60)) looper = false;
    delay(100);
    knob = enc_read();
    if ( (knob == 0) && (p.x > 24) && (p.x < 42) && (p.y > 45) && (p.y < 60) ) {
      knob = Get_Mouse_Wheel_Value();
    }
    if (knob != 0) {
      if (wpm > 4 && knob < 0)
        wpm--;
      if (wpm < 60 && knob > 0)
        wpm++;
      tft.fillRect(24, 48, 30, 24, ILI9341_BLACK);
      tft.setCursor(24, 48 );
      itoa(wpm, b, 10);
      tft.println(b);
      if ( (p.x > 24) && (p.x < 42) && (p.y > 45) && (p.y < 60) ) Update_Mouse();
    }
  }

  //save the setting
  cwSpeed = 600 / wpm;
  Put_SDEEPROM( CW_SPEED, wpm );
  tft.fillRect(24, 48, 30, 24, ILI9341_BLACK);
  itoa(wpm, b, 10);
  tftPrint(24, 48, b, Arial_18, ILI9341_WHITE, 0, ILI9341_WHITE );
  Update_Mouse();
}



void menuSetupCwTone( void ) {
  int knob = 0;
  bool looper = true;
  char buf[40];

  tone(CW_TONE, sideTone);
  sprintf(buf, "%04d", (int)sideTone);
  tft.setFont(Arial_16_Bold);
  tft.fillRect(221, 78, 38, 18, ILI9341_BLUE);
  tft.setTextColor(ILI9341_ORANGE);
  tft.setCursor(222, 78);
  tft.println(buf);

  while (Set_Left_Button_Pressed() == 1) Fake_Delay();
  while ( looper ) {
    if ( ts.touched() ) {
      Get_Touch(true);
      if ( (p.x > 140) && (p.x < 300) && (p.y > 78) && (p.y < 98)) {
        looper = false;
        Update_Mouse();
      }
    }
    if (Set_Left_Button_Pressed() == 1) looper = false;
    delay(100);
    knob = enc_read();
    if ( (knob == 0) && (p.x > 140) && (p.x < 300) && (p.y > 78) && (p.y < 98) ) {
      knob = Get_Mouse_Wheel_Value();
    }
    if (knob > 0 && sideTone < 1050)
      sideTone += 10;
    else if (knob < 0 && sideTone > 300 )
      sideTone -= 10;
    else
      continue; //don't update the frequency or the display
    tone(CW_TONE, sideTone);
    sprintf(buf, "%04d", (int)sideTone);
    tft.fillRect(221, 78, 38, 18, ILI9341_BLUE);
    tft.setCursor(222, 78 );
    tft.println(buf);
    if ((p.x > 227) && (p.x < 264) && (p.y > 78) && (p.y < 98))Update_Mouse();
  }
  noTone(CW_TONE);
  //save the setting
  Put_SDEEPROM( CW_SIDETONE, sideTone );

}

void Process_Volts( void ) {
  int temp;
  if (Volts_Active) {
    Volts_Active = false;
    //   printLineXI(0, 0, (char *)"     ");
  } else {
    Volts_Active = true;
  }
  temp = Volts_Active;
  Put_SDEEPROM( VOLTS, temp );
  delay(200);


}



void Process_Idle( void ) {

  switch ( Keyboard_Input_Buffer[OutKeyIndex] ) {

    case CTL_c:
      if ( (ubitx_mode == 2) || (ubitx_mode == 3) || (ubitx_mode == 6)) {
        Keyboard_Mode = KEY_CW;
        strcpy(Key_buffer2, ">");
        tftPrint(2,
                 224,
                 Key_buffer2,
                 Arial_12,
                 ILI9341_WHITE,
                 0,
                 ILI9341_WHITE );
        tft.fillRect(20, 220, 290, 24, ILI9341_BLACK);
        strcpy(Key_buffer1, "");
        strcpy(Key_buffer2, "");
        Key_Displayed = true;
        Yellow_Key_Displayed = false;
        Red_Key_Displayed = false;
        Key_Freq = 0;
      }
      break;

    case CTL_d:
      Current_Band = Current_Band - 1;
      if ( Current_Band < 0 )Current_Band = 0;
      Set_Band();
      break;

    case CTL_f:
      Keyboard_Mode = KEY_FREQ;
      strcpy(Key_buffer2, "F=   ");
      tftPrint(2,
               224,
               Key_buffer2,
               Arial_12,
               ILI9341_WHITE,
               0,
               ILI9341_WHITE );
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = 0;
      break;

    case CTL_m:
      Display_RIT();
      ubitx_mode++;
      if ( ubitx_mode > 6) ubitx_mode = 0;
      Put_SDEEPROM( MODE, ubitx_mode );
      delay(200);
      setFrequency(frequency);
      Display_Mode();
      updateDisplay();
      break;

    case CTL_i:
      if ( IAMBIC == 0 ) {
        SetIambicB();
      } else {
        SetIambicA();
      }
      tft.fillRect(0, 78, 60, 15, ILI9341_WHITE);
      tft.setFont(Arial_10_Bold);
      tft.setCursor(2, 80 );
      tft.setTextColor(ILI9341_BLACK );
      if ( IAMBIC == 0 ) {
        tft.println((char*)"IambicA");
      } else {
        tft.println((char*)"IambicB");
      }
      delay(2000);
      tft.fillRect(0, 78, 60, 15, ILI9341_BLACK);
      break;

    case CTL_l:
      if ( Tune_Lock == 0 ) {
        Display_Lock_ON();
      } else {
        Display_Lock();
      }
      break;

    case CTL_p:
      if ( PDLSWAP == 0 ) {
        SetPaddleSwapR();
      } else {
        SetPaddleSwapN();
      }
      Put_SDEEPROM( PDL_SWAP_VAL, PDLSWAP );
      tft.fillRect(0, 78, 64, 15, ILI9341_WHITE);
      tft.setFont(Arial_10_Bold);
      tft.setCursor(1, 80 );
      if ( PDLSWAP == 0 ) {
        tft.setTextColor(ILI9341_BLACK );
        tft.println((char*)"Norm Pdl");
      } else {
        tft.setTextColor(ILI9341_RED );
        tft.println((char*)"Rev   Pdl");
      }
      delay(2000);
      tft.fillRect(0, 78, 64, 15, ILI9341_BLACK);
      break;

    case CTL_v:
      menuVfoToggle();
      break;

    case CTL_r:
      if ( splitOn == 0 ) {
        if (ritOn)Display_RIT();
        else Display_RIT_ON();
      }
      break;

    case CTL_s:
      if ( !ritOn) menuSplitToggle();
      break;

    case CTL_w:
      Keyboard_Mode = KEY_WPM;
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = wpm;
      Display_Current_Speed(true);
      break;

    case CTL_t:
      Keyboard_Mode = KEY_TONE;
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = sideTone;
      tft.fillRect(0, 78, 40, 16, ILI9341_BLACK);
      tft.setFont(Arial_16_Bold);
      tft.setTextColor(ILI9341_WHITE );
      tft.setCursor(0, 78 );
      tft.println("T");
      break;

    case CTL_u:
      Current_Band = Current_Band + 1;
      if ( Current_Band > 7 )Current_Band = 7;
      Set_Band();
      break;

    case CTL_IF:
      // printLineXI(2, 0, (char*)">");
      delay(1000);
      //  printLineXI(2, 0, (char*)" ");
      break;

    default:
      // sprintf(Key_buffer2, "key=%x", Keyboard_Input_Buffer[OutKeyIndex]);
      //printLineXI(1, 0, Key_buffer2);
      break;
  }
  KeyIndex = 0;
  SendKeyIndex = 0;
}


void Process_Speed( void ) {


  tft.setFont(Arial_18);
  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 5 &&  Key_Freq <= 50 ) {
      wpm = Key_Freq;
      cwSpeed = 600 / wpm;
      Put_SDEEPROM( CW_SPEED, wpm );
      Keyboard_Mode = KEY_IDLE;
    } else {
      tft.setTextColor(ILI9341_RED);
      tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
      tft.setCursor(24, 48 );
      tft.println(Key_buffer2);
      delay(1000);
    }
    Display_Current_Speed(true);
    delay(1000);
    Display_Mode();

    Keyboard_Mode = KEY_IDLE;
  } else if ( Keyboard_Value == 0x1b ) {
    itoa(wpm, Key_buffer2, 10);
    tft.setTextColor(ILI9341_WHITE);
    tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
    tft.setCursor(24, 48 );
    tft.println(Key_buffer2);
    Keyboard_Mode = KEY_IDLE;
    //  updateDisplay();
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        tft.setTextColor(ILI9341_ORANGE);
        tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
        tft.setCursor(24, 48 );
        tft.println(Key_buffer2);
      }
      Key_Displayed = true;
    }
  } else {
    if (isdigit(Keyboard_Value)) {
      SendKeyIndex = SendKeyIndex + 1;
      if ( SendKeyIndex > 2 )SendKeyIndex = 2;
      else {
        sprintf(Key_buffer1, "%c", Keyboard_Value);
        strcat(Key_buffer2, Key_buffer1);
        Key_Freq = atol(Key_buffer2);
        if ( !Key_Displayed ) {
          tft.setTextColor(ILI9341_ORANGE);
          tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
          tft.setCursor(24, 48 );
          tft.println(Key_buffer2);
        }
        Key_Displayed = true;
      }
    }
  }
}


void Process_Tone( void ) {


  tft.setFont(Arial_16_Bold);
  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 100 &&  Key_Freq <= 990 ) {
      sideTone = Key_Freq;
      Put_SDEEPROM( CW_SIDETONE, sideTone );
      Keyboard_Mode = KEY_IDLE;
    } else {
      tft.fillRect(14, 78, 42, 18, ILI9341_BLACK);
      tft.setTextColor(ILI9341_RED);
      tft.setCursor(14, 78 );
      tft.println(Key_buffer2);
      delay(1000);
    }
    delay(500);
    tft.fillRect(0, 78, 56, 18, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else if ( Keyboard_Value == 0x1b ) {
    tft.fillRect(14, 78, 42, 18, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        tft.fillRect(14, 78, 42, 18, ILI9341_BLACK);
        tft.setTextColor(ILI9341_ORANGE);
        tft.setCursor(14, 78 );
        tft.println(Key_buffer2);
      }
      Key_Displayed = true;
    }
  } else {
    if (isdigit(Keyboard_Value)) {
      SendKeyIndex = SendKeyIndex + 1;
      if (SendKeyIndex > 3 )SendKeyIndex = 3;
      else {
        sprintf(Key_buffer1, "%c", Keyboard_Value);
        strcat(Key_buffer2, Key_buffer1);
        Key_Freq = atol(Key_buffer2);
        if ( !Key_Displayed ) {
          tft.fillRect(14, 78, 42, 18, ILI9341_BLACK);
          tft.setTextColor(ILI9341_ORANGE);
          tft.setCursor(14, 78 );
          tft.println(Key_buffer2);
        }
        Key_Displayed = true;
      }
    }
  }
}


int Local_Key_Send_Buffer_Count;


void Process_CW( void ) {
  int i;
  KeyIndex = 0;

  noInterrupts();
  Local_Key_Send_Buffer_Count = Key_Send_Buffer_Count;
  interrupts();

  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x1b ) {
    noInterrupts();
    Key_Send_Buffer_Count = 0;
    Key_Send_Buffer_IN_Index = 0;
    Key_Send_Buffer_OUT_Index = 0;
    sendKey(toupper(0));
    interrupts();
    Yellow_Key_Displayed = false;
    Red_Key_Displayed = false;
    tft.fillRect(2, 220, 317, 24, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else if ( Keyboard_Value == 0x0A ) {
    sendKey(toupper(0));
    tft.fillRect(2, 220, 317, 24, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else {
    if ( Local_Key_Send_Buffer_Count > (MAX_KEYING_BUFFER - 25)) {
      if (Yellow_Key_Displayed == false )Yellow_Key_Displayed = true;
      tft.fillRect(20, 220, 299, 24, ILI9341_YELLOW);
      tftPrint(20,
               223,
               Key_buffer1,
               Arial_12,
               ILI9341_RED,
               0,
               ILI9341_WHITE );
    } else {
      SendKeyIndex = SendKeyIndex + 1;
      sprintf(Key_buffer0, "%c", toupper(Keyboard_Value));
      if ( SendKeyIndex > MAXKYBDENTRY  ) {
        for (i = 0; i < MAXKYBDENTRY - 1; i++)Key_buffer1[i] = Key_buffer1[i + 1];
        SendKeyIndex = MAXKYBDENTRY;
        Key_buffer1[MAXKYBDENTRY - 1] = Key_buffer0[0];
        Key_buffer1[MAXKYBDENTRY] = 0;
      } else strcat(Key_buffer1, Key_buffer0);
      tft.fillRect(20, 220, 290, 24, ILI9341_BLACK);
      tftPrint(20,
               223,
               Key_buffer1,
               Arial_12,
               ILI9341_WHITE,
               0,
               ILI9341_WHITE );
      if (Keyboard_Value == ' ')Keyboard_Value = 1;
      sendKey(toupper(Keyboard_Value));
    }
  }
}


void Process_Freq( void ) {

  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];

  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 400000 && Key_Freq <= 30000000 ) {
      frequency = Key_Freq;
      if ( vfoActive == VFO_A ) vfoA = frequency;
      else vfoB = frequency;
      setFrequency(frequency);
      updateDisplay();
    } else {
      tft.fillRect(24, 224, 100, 24, ILI9341_BLACK);
      tftPrint(24,
               224,
               (char*)"Bad input  ",
               Arial_12,
               ILI9341_RED,
               0,
               ILI9341_WHITE );
      delay(1000);
    }
    tft.fillRect(2, 224, 100, 24, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else if ( Keyboard_Value == 0x1b ) {
    tft.fillRect(2, 224, 100, 24, ILI9341_BLACK);
    Keyboard_Mode = KEY_IDLE;
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        tft.fillRect(24, 224, 100, 24, ILI9341_BLACK);
        tftPrint(24,
                 224,
                 Key_buffer2,
                 Arial_12,
                 ILI9341_WHITE,
                 0,
                 ILI9341_WHITE );
      }
      Key_Displayed = true;
    }
  } else {
    if (isdigit(Keyboard_Value)) {
      SendKeyIndex = SendKeyIndex + 1;
      sprintf(Key_buffer1, "%c", Keyboard_Value );
      strcat(Key_buffer2, Key_buffer1);
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed )      tftPrint(24,
                                            224,
                                            Key_buffer2,
                                            Arial_12,
                                            ILI9341_WHITE,
                                            0,
                                            ILI9341_WHITE );

      Key_Displayed = true;
    }
  }
}



void Process_Keyboard( void ) {

  //Serial.println(Key_Send_Buffer_Count);

  if ( KeyIndex == 1 ) {

    Key_Displayed = false;
    Keyboard_Value = Keyboard_Input_Buffer[0];
    KeyIndex = 0;

    switch ( Keyboard_Mode ) {

      case  ( KEY_CW ):
        Process_CW();
        break;

      case  ( KEY_IDLE ):
        Process_Idle();
        break;

      case  ( KEY_FREQ ):
        Process_Freq();
        break;

      case  ( KEY_TONE ):
        Process_Tone();
        break;

      case  ( KEY_WPM ):
        Process_Speed();
        break;

    }
  }
}
