/**
   ubitx_ui.cpp
   The user interface of the uBITX consists of the encoder, the push-button on top of it
   and the 16x2 LCD display.
   The upper line of the display is constantly used to display frequency and status
   of the radio. Occasionally, it is used to provide a two-line information that is
   quickly cleared up.
*/

//
#include <Arduino.h>
#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <XPT2046_Touchscreen.h>
#include <SPIN.h>
#include <SPI.h>
#include <SD.h>
#include "ubitx.h"


extern ILI9341_t3n tft;
extern bool Put_SDEEPROM( int index, long Value );
extern void Display_Mode(void);
extern void menuVfoToggle(void);
extern void setFrequency(unsigned long f);
extern void Get_Touch( bool wait );
extern XPT2046_Touchscreen ts;
extern bool Set_Left_Button_Pressed(void);
extern void Fake_Delay(void);
extern void Process_Reboot(void);
extern void Remove_Mouse( void );
extern void Display_Smeter(void);


extern void Init_Screen(void);
extern void Process_Memory_Tables(void);
extern void SetIambicA();
extern void SetIambicB();
extern int wpm;
extern bool Smeter_Active;

extern char c[21], b[21];
extern unsigned long frequency;
extern unsigned long vfoA, vfoB;
extern char inTx;
extern char ritOn;
extern char splitOn;
extern char vfoActive;
extern int Freq_Underline[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern long ritOffset;
extern bool Volts_Active;
extern unsigned volatile char IAMBIC;
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Converted_Ubitx_Voltage;
extern unsigned long Volt_Divider;
extern bool Cat_Lock;
extern bool CAT_S_Meter_Active;
//extern bool CW_Calibrated;
//extern bool CW_Filter_Present;
extern int Keyboard_Mode;

char Time_Buffer[12];
bool CAT_Volts_Active = false;

int Mem_Number = 0;
int Mem_Table = 3;

int isUSB = 0;

bool VFO_A_Displayed = false;
bool VFO_B_Displayed = false;

char ubitx_mode_text[7][5] = { "USB", "LSB", "CW ", "CWR", "SWU", "SWL" , "CWP"};
int   Working_Copy_Modes[10];
unsigned long Working_Copy_Frequencies[10];
unsigned long Band_Table[2][8];

extern bool CW_Active;

char Fill_Text[12] = {'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'};
char Last_VFO_A_Text[12] = {'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'};
char Last_VFO_B_Text[12] = {'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'};



volatile int SmeterReadValue, LastSmeter, SMeter_Value, smeterInterval;

/////////////////////////////////////////////////////////////////////////////





void tftPrint(int x,
              int y,
              char* Text,
              ILI9341_t3_font_t font = Arial_14_Bold,
              int fontColor = ILI9341_WHITE,
              bool textframed = false,
              int frameColor = ILI9341_WHITE ) {

  tft.setFont(font);
  tft.setTextColor(fontColor);
  tft.setCursor(x, y);
  tft.print(Text);


}





void Place_The_Cursor( void) {
  int Freq_Incr_Index[6] = {274, 252, 216, 192, 168, 132};

  // draw freq incr pointers
  tft.fillRect(132, 189, 170, 3, ILI9341_BLACK);
  tft.fillRect(132, 111, 170, 3, ILI9341_BLACK);

  if ( vfoActive == VFO_B ) {
    //VFO B
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 111, 20, ILI9341_GREEN);
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 112, 20, ILI9341_GREEN);
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 113, 20, ILI9341_GREEN);
  } else {
    //VFO A
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 189, 20, ILI9341_GREEN);
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 190, 20, ILI9341_GREEN);
    tft.drawFastHLine(Freq_Incr_Index[Freq_Index], 191, 20, ILI9341_GREEN);
  }


}





void Display_S_Meter( void ) {
  char SmeterText[16];
  uint8_t oldSREG;
  int Normal_Smeter_Length;
  int Red_Smeter_Length;
  int  level;

  if ( Smeter_Active ) {
    if ( Keyboard_Mode != KEY_CW ) {
      oldSREG = SREG;
      cli();
      SmeterReadValue = analogRead(SMETERREADPIN);   // read value of signal 0 - 1023
      level = SmeterReadValue;  // kill time
      SmeterReadValue = analogRead(SMETERREADPIN);   // read value of signal 0 - 1023
      SREG = oldSREG;


      level = SmeterReadValue / 40;

      if (level > 30) level = 40;
      else if (level > 20 ) level = 30;
      else if (level > 10 ) level = 20;
      else if (level > 7  ) level = 9;
      else if (level > 5  ) level = 7;
      else if (level > 3  ) level = 5;
      else if (level > 1  ) level = 3;
      else if (level > 0  ) level = 1;
      else level = 0;

      /*      level = SmeterReadValue / 40;

            if (level >= 30) level = 40;
            if (level >= 20 && level < 30) level = 30;
            if (level >= 10 && level < 20) level = 20;
            if (level >= 7 && level < 10) level = 9;
            if (level >= 5 && level < 8) level = 7;
            if (level >= 3 && level < 6) level = 5;
            if (level >= 1 && level < 4) level = 3;
            if (level >= 0 && level < 2) level = 1;
      */
      if (level != LastSmeter ) {
        tft.fillRect(96, 131, 204, 9, ILI9341_WHITE);
        if ( level <= 9 ) {
          Normal_Smeter_Length = level * 13;
          tft.fillRect(96, 131, Normal_Smeter_Length, 9, ILI9341_GREEN);
        } else {
          tft.fillRect(96, 131, 120, 9, ILI9341_GREEN);
          // red zone   40   130  -> 40y = 130x    y= 130/40
          Red_Smeter_Length = (level) * 2;
          if ( Red_Smeter_Length > 130 )Red_Smeter_Length = 130;
          tft.fillRect(216, 131, Red_Smeter_Length, 9, ILI9341_RED);
        }
        if ( CAT_S_Meter_Active == true ) {
          sprintf(SmeterText, "SM0%03d;", (int)level);
          Serial.write(SmeterText);
          delay(10);
        }
        LastSmeter = level;
        Place_The_Cursor();

      }
    }
  }
}


void Display_Voltage( void ) {
  char local_b[12];
  int i;

  for (i = 0; i < 12; i++)Time_Buffer[i] = 0;
  // Display VCC volts
  if ( (Volts_Active) && (!inTx )) {
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)150);
    Converted_Ubitx_Voltage = Converted_Ubitx_Voltage  / Volt_Divider;
    ultoa(Converted_Ubitx_Voltage, local_b, 10);
    if ( strlen(local_b) == 1) {
      strcpy(Time_Buffer, "00");
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, b, 1);
    } else if ( strlen(local_b) == 2) {
      strcpy(Time_Buffer, "0");
      strncat(Time_Buffer, local_b, 1);
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, &local_b[1], 1);
    } else {
      strncpy(Time_Buffer, local_b, 2);
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, &local_b[2], 1);
    }
    strcat(Time_Buffer, "V ");
    Time_Buffer[strlen(Time_Buffer)] = 0;
    Current_Ubitx_Voltage = Ubitx_Voltage;
    // draw volts
    tft.setFont(Arial_14_Bold);
    if ( Converted_Ubitx_Voltage > 116 )tft.setTextColor(ILI9341_WHITE);
    else tft.setTextColor(ILI9341_RED);
    tft.fillRect(266, 48, 44, 20, ILI9341_BLACK);
    tft.setCursor(266, 48);
    tft.println(Time_Buffer);
  }
  if ( Cat_Lock && CAT_Volts_Active && Volts_Active ) {

    sprintf(local_b, "uVR%s;", Time_Buffer);
    local_b[strlen(local_b)] = 0;
    Serial.write(local_b);
    delay(10);

  }
}


void Display_RIT_Data( void) {
  unsigned long temp;

  if ( ritOffset < 0 ) {
    strcpy(c, " -");
    temp = 0 - ritOffset;
  } else {
    strcpy(c, " +");
    temp = ritOffset;
  }
  ultoa(temp / 100, b, DEC);
  if ( strlen(b) == 1) {
    strcat(c, "0.");
    strncat(c, &b[0], 1);
  }
  if ( strlen(b) == 2) {
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 1);
  }
  strcat(c, " ");

  tft.fillRect(196, 196, 800, 26,  ILI9341_BLACK);
  tft.setFont(Arial_24);
  tft.setCursor(196, 196);
  tft.setTextColor(ILI9341_ORANGE);
  tft.print(c);
}

char Memory_Tables[4][4] = {"CW ", "SSB", "DIG", "USR"};
extern int Memory_Table_Select;



void Display_Mem_Selection( void ) {
  char fred[21];

  tft.drawRect(2, 2, 160, 42, ILI9341_WHITE);
  tft.fillRect(3, 3, 158, 40, ILI9341_BLUE);

  tft.setFont(Arial_12_Bold);
  tft.setTextColor(ILI9341_BLACK);
  tft.fillRect(5, 5, 38, 16, ILI9341_GREEN);
  tft.setCursor(9, 7);
  sprintf(fred, "%3s", Memory_Tables[Memory_Table_Select]);
  tft.println(fred);
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(52, 6);
  tft.println(":");
  sprintf(fred, "%1d",  Mem_Number);
  tft.fillRect(60, 5, 20, 16, ILI9341_GREEN);
  tft.setCursor(64, 6);
  tft.setTextColor(ILI9341_BLACK);
  tft.println(fred);
  tft.setTextColor(ILI9341_BLACK);
  tft.fillRect(88, 5, 24, 16, ILI9341_ORANGE);
  tft.setCursor(90, 6);
  tft.println("Ld");
  tft.fillRect(118, 5, 24, 16, ILI9341_ORANGE);
  tft.setCursor(120, 6);
  tft.println("St");

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));
  ultoa(Working_Copy_Frequencies[Mem_Number], b, DEC);
  if (strlen(b) < 7) {
    strcat(&c[0], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);
  } else if (strlen(b) < 8) {
    strcat(&c[0], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 2);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);
  }
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(6, 26);
  tft.println(ubitx_mode_text[Working_Copy_Modes[Mem_Number]]);
  tft.setCursor(64, 26);
  tft.println(c);
}


////////////////////////////////////////////////////////////////////////////////

void updateDisplay(void) {
  int i;
  int index;
  char tmp[2];
  char ltmp[10];

  if ( !ritOn ) {

    memset(c, 0, sizeof(c));
    memset(b, 0, sizeof(b));

    if ( (vfoActive == VFO_B) || ( VFO_B_Displayed == false ) ) {
   //   tft.fillRect(106, 76, 192, 42, ILI9341_BLACK);
      if ( vfoActive == VFO_B ) {
        tft.setTextColor(ILI9341_ORANGE);
      } else {
        tft.setTextColor(ILI9341_LIGHTGREY);
        VFO_B_Displayed = true;
      }

      ultoa(vfoB, b, DEC);
      if (strlen(b) < 7) {
        strcat(&c[0], "00.");
        strncat(c, &b[0], 3);
        strcat(c, ".");
        strncat(c, &b[3], 2);
      } else if (strlen(b) < 8) {
        strcat(&c[0], "0");
        strncat(c, &b[0], 1);
        strcat(c, ".");
        strncat(c, &b[1], 3);
        strcat(c, ".");
        strncat(c, &b[4], 2);
      } else {
        strncat(c, b, 2);
        strcat(c, ".");
        strncat(c, &b[2], 3);
        strcat(c, ".");
        strncat(c, &b[5], 2);
      }

      tft.setFont(Arial_32_Bold);
      strcpy(ltmp, c);
      for ( i = 0; i < 9; i++) {
        index = 106 + (i * 24);
        if ( i > 2 )index = 106 + (i * 24) - 12;
        if ( i > 6 )index = 106 + (i * 24) - 24;
        tft.setCursor(index, 76);

        if ( Last_VFO_B_Text[i] != c[i] ) {
          strncpy(tmp, &c[i], 1);
          tmp[1] = 0;
          tft.fillRect(index, 76, 24, 42, ILI9341_BLACK);
          tft.print(tmp);
        }
      }
      strcpy(Last_VFO_B_Text, ltmp);

    }
  }
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));


  if ( !ritOn ) {

    if ( (vfoActive == VFO_A) || ( VFO_A_Displayed == false ) ) {
      if ( vfoActive == VFO_A ) {
        tft.setTextColor(ILI9341_ORANGE);
      } else {
        tft.setTextColor(ILI9341_LIGHTGREY);
        VFO_A_Displayed = true;
      }
      ultoa(vfoA, b, DEC);
      if (strlen(b) < 7) {
        strcpy(&c[0], "00.");
        strncat(c, &b[0], 3);
        strcat(c, ".");
        strncat(c, &b[3], 2);
      } else if (strlen(b) < 8) {
        strcpy(&c[0], "0");
        strncat(c, &b[0], 1);
        strcat(c, ".");
        strncat(c, &b[1], 3);
        strcat(c, ".");
        strncat(c, &b[4], 2);
      } else {
        strncpy(c, b, 2);
        strcat(c, ".");
        strncat(c, &b[2], 3);
        strcat(c, ".");
        strncat(c, &b[5], 2);
      }

      tft.setFont(Arial_32_Bold);
      strcpy(ltmp, c);
      for ( i = 0; i < 9; i++) {
        index = 106 + (i * 24);
        if ( i > 2 )index = 106 + (i * 24) - 12;
        if ( i > 6 )index = 106 + (i * 24) - 24;
        tft.setCursor(index, 154);

        if ( Last_VFO_A_Text[i] != c[i] ) {
          strncpy(tmp, &c[i], 1);
          tmp[1] = 0;
          tft.fillRect(index, 144, 24, 42, ILI9341_BLACK);
          tft.print(tmp);
        }
      }
      strcpy(Last_VFO_A_Text, ltmp);

    }
  } else Display_RIT_Data();

  //if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR)) {
    //if (!CW_Calibrated && CW_Filter_Present) {
    //  tft.setTextColor(ILI9341_RED, ILI9341_WHITE);
    //  tft.setFont(Arial_16_Bold);
    //  tft.setCursor(150, 222);
    //  tft.println((char*)"*** DO CW CAL");
    //}
  //}

  Place_The_Cursor();



}

///////////////////////////////////////////  pulldowns
extern TS_Point p;


void Remove_Channel_Menu( void ) {
  int Current_Mem_Number;

  Current_Mem_Number = Mem_Number;
  tft.fillRect(2, 2, 180, 210, ILI9341_BLACK);
  Init_Screen();

  if    ( ( (p.x > 4) && (p.x < 175) && (p.y > 5) && (p.y < 25)))   Mem_Number = 0;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 25) && (p.y < 45))  Mem_Number = 1;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 45) && (p.y < 65))  Mem_Number = 2;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 65) && (p.y < 85))  Mem_Number = 3;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 85) && (p.y < 110))  Mem_Number = 4;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 110) && (p.y < 135))  Mem_Number = 5;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 135) && (p.y < 160))  Mem_Number = 6;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 160) && (p.y < 180))  Mem_Number = 7;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 180) && (p.y < 205))  Mem_Number = 8;
  else if ( (p.x > 4) && (p.x < 175) && (p.y > 205) && (p.y < 225))  Mem_Number = 9;
  else Mem_Number = Mem_Number;
  if ( (Mem_Number < 0) || (Mem_Number > 9) )Mem_Number = Current_Mem_Number;
  tft.fillRect(2, 2, 180, 230, ILI9341_BLACK);
  Init_Screen();
  Display_Mem_Selection();
  menuVfoToggle();
  menuVfoToggle();
}

void Display_Channel_Menu( void ) {
  int i;
  char fred[21];


  tft.drawRect(2, 2, 180, 230, ILI9341_WHITE);
  tft.fillRect(3, 3, 178, 228, ILI9341_BLACK);

  tft.setFont(Arial_14);
  tft.setTextColor(ILI9341_ORANGE);

  for (i = 0; i < 10; i++) {
    tft.setCursor(8, 10 + (i * 22));
    sprintf(fred, "%1d:",  i);
    tft.println(fred);
    tft.setCursor(28, 10 + (i * 22));
    tft.println(ubitx_mode_text[Working_Copy_Modes[i]]);

    memset(c, 0, sizeof(c));
    memset(b, 0, sizeof(b));
    ultoa(Working_Copy_Frequencies[i], b, DEC);
    if (strlen(b) < 7) {
      strcat(&c[0], "00.");
      strncat(c, &b[0], 3);
      strcat(c, ".");
      strncat(c, &b[3], 2);
    } else if (strlen(b) < 8) {
      strcat(&c[0], "0");
      strncat(c, &b[0], 1);
      strcat(c, ".");
      strncat(c, &b[1], 3);
      strcat(c, ".");
      strncat(c, &b[4], 2);
    } else {
      strncat(c, b, 2);
      strcat(c, ".");
      strncat(c, &b[2], 3);
      strcat(c, ".");
      strncat(c, &b[5], 2);
    }
    tft.setCursor(80, 10 + (i * 22));
    tft.println(c);
  }
}


void Remove_Table_Menu( void ) {

  tft.fillRect(2, 2, 54, 80, ILI9341_BLACK);
  Init_Screen();

  if    ( ( (p.x > 5) && (p.x < 60) && (p.y > 2) && (p.y < 20))) Memory_Table_Select = 0;
  else if ( (p.x > 5) && (p.x < 60) && (p.y > 20) && (p.y < 30))  Memory_Table_Select = 1;
  else if ( (p.x > 5) && (p.x < 60) && (p.y > 40) && (p.y < 55))  Memory_Table_Select = 2;
  else if ( (p.x > 5) && (p.x < 60) && (p.y > 60) && (p.y < 75))  Memory_Table_Select = 3;

  Process_Memory_Tables();

}

void Display_Table_Menu( void ) {
  char fred[21];

  tft.drawRect(2, 2, 54, 80, ILI9341_WHITE);
  tft.fillRect(3, 3, 52, 78, ILI9341_BLACK);
  tft.setFont(Arial_14);
  tft.setTextColor(ILI9341_ORANGE);
  tft.setCursor(8, 8);
  sprintf(fred, "%3s", Memory_Tables[0]);
  tft.println(fred);

  tft.setCursor(8, 26);
  sprintf(fred, "%3s", Memory_Tables[1]);
  tft.println(fred);

  tft.setCursor(8, 44);
  sprintf(fred, "%3s", Memory_Tables[2]);
  tft.println(fred);

  tft.setCursor(8, 62);
  sprintf(fred, "%3s", Memory_Tables[3]);
  tft.println(fred);

}





void Remove_Mode_Menu( void ) {
  int NotHit = 1;
  int temp_mode = -1;

  while ( NotHit ) {
    while ((!ts.touched()) && (Set_Left_Button_Pressed() == 0) ) {
      Fake_Delay();
      Fake_Delay();
    }
    if (ts.touched()) Get_Touch( true );
    else while (Set_Left_Button_Pressed() == 1 )Fake_Delay();

    if    ( ( (p.x > 195) && (p.x < 260) && (p.y > 25) && (p.y < 49)))  {
      temp_mode = 0;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 50) && (p.y < 79))   {
      temp_mode = 1;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 80) && (p.y < 99))  {
      temp_mode = 2;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 100) && (p.y < 129)) {
      temp_mode = 3;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 130) && (p.y < 154)) {
      temp_mode = 4;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 155) && (p.y < 180)) {
      temp_mode = 5;
      NotHit = false;
    }
    else if ( (p.x > 195) && (p.x < 260) && (p.y > 180) && (p.y < 210)) {
      temp_mode = 6;
      NotHit = false;
    }
    else {
      Remove_Mouse();
      NotHit = false;
    }
  }
  //    Remove_Mouse();
  if ( temp_mode >= 0) {
    ubitx_mode = temp_mode;
    Put_SDEEPROM( MODE, ubitx_mode );
  }
  tft.fillRect(186, 35, 78, 172, ILI9341_BLACK);
  Init_Screen();
  LastSmeter = 0;
  Display_S_Meter();
  Display_Mode();
  setFrequency(frequency);
  VFO_B_Displayed = false;
  VFO_A_Displayed = false;
  strcpy(Last_VFO_A_Text, Fill_Text);
  strcpy(Last_VFO_B_Text, Fill_Text);
  Display_Smeter();
  updateDisplay();

}


void Display_Mode_Menu( void ) {

  tft.drawRect(186, 35, 78, 172, ILI9341_WHITE);
  tft.fillRect(187, 36, 76, 170, ILI9341_BLACK);
  tft.setFont(Arial_20_Bold);
  tft.setTextColor(ILI9341_ORANGE);
  tft.setCursor(194, 38);
  tft.println((char*)"USB");

  tft.setCursor(194, 62);
  tft.println((char*)"LSB");

  tft.setCursor(194, 86);
  tft.println((char*)"CW ");

  tft.setCursor(194, 110);
  tft.println((char*)"CWR");

  tft.setCursor(194, 134);
  tft.println((char*)"SWU");

  tft.setCursor(194, 158);
  tft.println((char*)"SWL");

  tft.setCursor(194, 182);
  tft.println((char*)"CWP");


}

void Display_Current_Speed(bool display_speed) {
  char buffr[24];

  tft.fillRect(6, 48, 46, 20, ILI9341_BLACK);

  if (display_speed ) {
    tft.setTextColor(ILI9341_WHITE);
    tft.setFont(Arial_18);
    tft.setCursor(6, 48 );
    tft.println("S");
    itoa(wpm, buffr, 10);
    tft.setCursor(24, 48 );
    tft.println(buffr);
  }

}


void Display_Mode( void ) {
  bool display_speed = false;
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  if ( ubitx_mode == MODE_USB )       strcpy(c, "USB");
  else if ( ubitx_mode == MODE_LSB   )strcpy(c, "LSB");
  else if ( ubitx_mode == MODE_CW    ) {
    strcpy(c, " CW");
    display_speed = true;
  }
  else if ( ubitx_mode == MODE_CWR   ) {
    strcpy(c, "CWR");
    display_speed = true;
  }
  else if ( ubitx_mode == MODE_SWU   )strcpy(c, "SWU");
  else if ( ubitx_mode == MODE_SWL   )strcpy(c, "SWL");
  else if ( ubitx_mode == MODE_CWP   ) {
    strcpy(c, "CWP");
    display_speed = true;
  }

  tft.setFont(Arial_24_Bold);
  tft.setTextColor(ILI9341_GREEN);
  tft.fillRect(188, 42, 78, 34, ILI9341_BLACK);
  tft.setCursor(188, 42);
  tft.println(c);

  Display_Current_Speed(display_speed);
  if ( display_speed )  isUSB = 0;
  else   isUSB = 1;

  if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR) || (ubitx_mode == MODE_CWP)) CW_Active = true;
  else CW_Active = false;


}



int enc_prev_state = -1;

/**
   The A0 And A1 are purely analog lines on the Arduino Nano
   These need to be pulled up externally using two 10K resistors

   There are excellent pages on the Internet about how these encoders work
   and how they should be used. We have elected to use the simplest way
   to use these encoders without the complexity of interrupts etc to
   keep it understandable.

   The enc_state returns a two-bit number such that each bit reflects the current
   value of each of the two phases of the encoder

   The enc_read returns the number of net pulses counted over 50 msecs.
   If the puluses are -ve, they were anti-clockwise, if they are +ve, the
   were in the clockwise directions. Higher the pulses, greater the speed
   at which the enccoder was spun
*/


int enc_read( void ) {
  int result = 0;
  byte newState;
  int enc_speed = 0;

  unsigned long previousMillis = millis();
  unsigned long stop_by = 50;
  unsigned long currentMillis = millis();

  while ((currentMillis - previousMillis) < stop_by) { // check if the previous state was stable
    newState = (digitalRead(ENC_A) == HIGH ? 1 : 0) + (digitalRead(ENC_B) == HIGH ? 2 : 0); // Get current state

    if (newState != enc_prev_state)
      delay (1);

    // if ( (digitalRead(ENC_A) == HIGH ? 1 : 0) + (digitalRead(ENC_B) == HIGH ? 2 : 0) != newState || newState == enc_prev_state) {
    if ( (digitalRead(ENC_A)) + (digitalRead(ENC_B) * 2) != newState || newState == enc_prev_state) {
      currentMillis = millis();
      continue;
    } else {
      //these transitions point to the encoder being rotated anti-clockwise
      if ((enc_prev_state == 0 && newState == 2) ||
          (enc_prev_state == 2 && newState == 3) ||
          (enc_prev_state == 3 && newState == 1) ||
          (enc_prev_state == 1 && newState == 0)) {
        //result--;
        result++;
      }
      //these transitions point o the enccoder being rotated clockwise
      if ((enc_prev_state == 0 && newState == 1) ||
          (enc_prev_state == 1 && newState == 3) ||
          (enc_prev_state == 3 && newState == 2) ||
          (enc_prev_state == 2 && newState == 0)) {
        //result++;
        result--;
      }
      enc_prev_state = newState; // Record state for next pulse interpretation
      enc_speed++;
      delay(1);
      currentMillis = millis();
    }
  }


  return (result);
}
