/**

   This module handles User Interface widgets
   Processes Touch and Mouse operations

  #include <Arduino.h>



*/

#include <i2c_t3.h>
#include "USBHost_t36.h"

#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <SPIN.h>
#include <XPT2046_Touchscreen.h>
#include <SPI.h>
//#include <SD.h>
//#include <EEPROM.h>
//#include <TimeLib.h>
#include "ubitx.h"

extern ILI9341_t3n tft;
extern XPT2046_Touchscreen ts;
extern MouseController uBITX_Mouse;

extern int  enc_read(void);
extern void Remove_Mouse(void);
extern void setFrequency(unsigned long xf);
extern void Display_PBT_Position(void);
extern void Display_PBT_OFF(void);
extern void Update_Mouse(void);
extern void Save_Mouse(void);
extern void updateDisplay(void);
extern void Display_RIT(void);
extern void menuVfoToggle(void);
extern void Display_Mode(void);
extern bool Put_SDEEPROM( int index, long Value );
extern void Truncate_Frequency ( void );
extern void Place_The_Cursor( void);
extern void Display_Trangle(int xstart, int ystart, int width, int height, uint16_t color);
extern void Set_Band( void);
extern void Display_Mode_Menu(void);
extern void Remove_Mode_Menu(void);
extern void Display_RIT_ON(void);
extern void menuSplitToggle( void );
extern void Display_Lock(void);
extern void Display_Lock_ON(void);
extern void Display_Table_Menu(void);
extern void Remove_Table_Menu(void);
extern void Display_Channel_Menu(void);
extern void Remove_Channel_Menu(void);
extern void Fake_Delay(void);
extern void Fake_Delay1(void);
extern void Display_Mem_Selection(void);
extern void Process_Memory_Load(void);
extern void menuCWSpeed(void);
extern void Process_Setup( void );

//extern bool PBT_Active;
extern bool PBT_ON;
extern long PBT_Value;
extern unsigned long frequency;
extern unsigned long vfoA, vfoB;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern bool ritOn;
extern int Current_Band;
extern char splitOn;
extern bool Tune_Lock;
extern int Mem_Number;
extern int Memory_Table_Select;

extern volatile int ubitx_mode;
extern int   Working_Copy_Modes[10];
extern unsigned long Working_Copy_Frequencies[10];
extern long ritOffset;
extern long ritOffsetMax;

//extern bool Rit_Encoder;

extern char vfoActive;


int CAL_Minx;
int CAL_Miny;
int CAL_Maxx;
int CAL_Maxy;

int16_t mouse_x = 10;
int16_t mouse_y = 100;
int16_t last_mouse_x = 10;
int16_t last_mouse_y = 100;

int mouse_left_button = 0;
int mouse_right_button = 0;
int Current_Mouse_Wheel = 0;

uint16_t Mouse_Data[200];

bool Mouse_Active = false;



void Display_Mouse( void) {
  if ( Mouse_Active ) tft.fillCircle(mouse_x + 5, mouse_y + 5, 4, ILI9341_YELLOW);
}

/////////////////////////////////// TOUCH

void Renew_Mouse(void);
void Reset_Mouse(void);

TS_Point p;

bool first;

int rawxx, rawyy;
int xx;
int yy;

void Get_Touch( bool wait = true ) {
#ifdef DEBUG
  char buffr[100];
#endif

  delay(100);
  p = ts.getPoint();
  if ( wait) {
    while ( ts.touched() ) delay(100);
  }
  rawxx = p.x;
  rawyy = p.y;
  xx = map(p.x, CAL_Minx, CAL_Maxx, 0, 319);
  yy = map(p.y, CAL_Miny, CAL_Maxy, 0, 239);
  p.x = xx;
  p.y = yy;

#ifdef DEBUG
  sprintf(buffr, "Rx=%03d Ry=%03d x=%03d y=%03d", rawxx, rawyy, p.x, p.y );
  Serial.println("");
  Serial.println("");
  Serial.println("");
  Serial.println("");
  Serial.println(buffr);
#endif

}


void Process_Memory_Store( void) {
  int Mem_Table_Ptr;
  int i;

  if (vfoActive == VFO_B)Working_Copy_Frequencies[Mem_Number] = vfoB;
  else Working_Copy_Frequencies[Mem_Number] = vfoA;
  Working_Copy_Modes[Mem_Number] = ubitx_mode;
  Mem_Table_Ptr = ( Memory_Table_Select + 1 ) * 100;
  for (i = 0; i < 10; i++) {
    Put_SDEEPROM( (Mem_Table_Ptr + (i * 4)), Working_Copy_Frequencies[i] );
    Put_SDEEPROM( (Mem_Table_Ptr + 40 + (i * 4)), Working_Copy_Modes[i] );
  }
  tft.setFont(Arial_12_Bold);
  tft.setTextColor(ILI9341_ORANGE);
  tft.fillRect(118, 5, 24, 16, ILI9341_BLACK);
  tft.setCursor(120, 6);
  tft.println("St");
  delay(1000);

  Display_Mem_Selection();
  updateDisplay();
}




//////////////////////////////////// M O U S E //////////////////////////////





void Process_Buttons( void ) {
  long rx_freq;

  //  if ( PBT_Active || PBT_ON ) {
  if ( PBT_ON ) {
    /*
      if ( PBT_Active ) {
      int knob = enc_read();
      if (knob != 0) {
        Remove_Mouse();
        if (knob < 0) {
          PBT_Value = PBT_Value - 30;
          if ( PBT_Value <  -2000) PBT_Value =  -2000;
          setFrequency(frequency);
          Display_PBT_Position();
        } else if (knob > 0) {
          PBT_Value = PBT_Value + 30;
          if ( PBT_Value > 2000) PBT_Value = 2000;
          setFrequency(frequency);
          Display_PBT_Position();
        }
      }
      }
    */
    if (ts.touched()) {
      Get_Touch(false);
      if ( (p.x > 0) && (p.x < 20) && (p.y > 125) && (p.y < 159)) {
        PBT_Value = PBT_Value - 15;
        if ( PBT_Value <  -2000) PBT_Value =  -2000;
        setFrequency(frequency);
        Display_PBT_Position();
        if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Renew_Mouse();
        delay(20);
        while ( ts.touched() && (p.x > 0) && (p.x < 20) && (p.y > 125) && (p.y < 159)) {
          PBT_Value = PBT_Value - 15;
          if ( PBT_Value <  -2000) PBT_Value =  -2000;
          setFrequency(frequency);
          Display_PBT_Position();
          if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Renew_Mouse();
          delay(20);
        }
        Display_Mouse();
      } else if ( (p.x > 30) && (p.x < 43) && (p.y > 125) && (p.y < 159)) {
        if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Renew_Mouse();
        PBT_Value = PBT_Value + 15;
        if ( PBT_Value > 2000) PBT_Value = 2000;
        setFrequency(frequency);
        Display_PBT_Position();
        if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Renew_Mouse();
        delay(20);
        while ( ts.touched() && (p.x > 30) && (p.x < 43) && (p.y > 125) && (p.y < 159)) {
          PBT_Value = PBT_Value + 15;
          if ( PBT_Value > 2000) PBT_Value = 2000;
          setFrequency(frequency);
          Display_PBT_Position();
          if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Renew_Mouse();
          delay(20);
        }
        Display_Mouse();
      }
    }
  }

  if (ts.touched()) {
    Get_Touch(false);

    if ( (p.x > 20) && (p.x < 25) && (p.y > 120) && (p.y < 145)) {
      if ( PBT_ON ) {
        Display_PBT_OFF();
        //        Rit_Encoder = false;
        //Update_Mouse();
        //      } else if ( PBT_ON ) {
        //        PBT_Active = true;
        //        Rit_Encoder = false;
      } else {
        // Remove_Mouse();
        Display_PBT_Position();
        PBT_ON = true;
        //        PBT_Active = true;
      }
      if ( (mouse_x > 0) && (mouse_x < 47) && (mouse_y > 112) && (mouse_y < 150)) Update_Mouse();
    } else if ( (p.x > 305) && (p.x < 320) && (p.y > 75) && (p.y < 125)) {
      if ( vfoActive == VFO_B ) rx_freq = vfoB;
      else rx_freq = vfoA;
      rx_freq += Freq_Incr[Freq_Index];
      if ( rx_freq <  400000l) rx_freq =  400000l;
      if ( rx_freq > 30000000l) rx_freq = 30000000l;
      if ( vfoActive == VFO_B )vfoB = rx_freq;
      else vfoA = rx_freq;
      frequency = rx_freq;
      setFrequency(frequency);
      updateDisplay();
      if (  first == true) {
        delay(1000);
        first = false;
      } else delay(200);


    } else if ( (p.x > 305) && (p.x < 320) && (p.y > 145) && (p.y < 195)) {
      if ( vfoActive == VFO_B ) rx_freq = vfoB;
      else rx_freq = vfoA;
      rx_freq -= Freq_Incr[Freq_Index];
      if ( rx_freq <  400000l) rx_freq =  400000l;
      if ( rx_freq > 30000000l) rx_freq = 30000000l;

      if ( vfoActive == VFO_B )vfoB = rx_freq;
      else vfoA = rx_freq;
      frequency = rx_freq;
      setFrequency(frequency);
      updateDisplay();
      if (  first == true) {
        delay(1000);
        first = false;
      } else delay(200);

      // vfo a
    } else if ( (p.x > 65) && (p.x < 90) && (p.y > 150) && (p.y < 200)) {
      //      PBT_Active = false;
      if (ritOn) {
        //        Rit_Encoder = true;
        Display_RIT();
      }
      if ( vfoActive == VFO_B ) {
        vfoActive = VFO_B;
        menuVfoToggle();
        Display_Mode();
        if ( frequency <=        4000000 )Current_Band = 0;
        else if ( frequency <=   7300000 )Current_Band = 1;
        else if ( frequency <=  10150000 )Current_Band = 2;
        else if ( frequency <=  14350000 )Current_Band = 3;
        else if ( frequency <=  18168000 )Current_Band = 4;
        else if ( frequency <=  21450000 )Current_Band = 5;
        else if ( frequency <=  24990000 )Current_Band = 6;
        else if (frequency <=   29700000 )Current_Band = 7;
      }

      // vfo b
    } else if ( (p.x > 65) && (p.x < 90) && (p.y > 70) && (p.y < 110)) {
      //      PBT_Active = false;
      if (ritOn) {
        //        Rit_Encoder = true;
        Display_RIT();
      }
      if ( vfoActive == VFO_A ) {
        vfoActive = VFO_A;
        menuVfoToggle();
        Display_Mode();
        if ( frequency <=        4000000 )Current_Band = 0;
        else if ( frequency <=   7300000 )Current_Band = 1;
        else if ( frequency <=  10150000 )Current_Band = 2;
        else if ( frequency <=  14350000 )Current_Band = 3;
        else if ( frequency <=  18168000 )Current_Band = 4;
        else if ( frequency <=  21450000 )Current_Band = 5;
        else if ( frequency <=  24990000 )Current_Band = 6;
        else if (frequency <=   29700000 )Current_Band = 7;
      }

      //  check for freq incr change on VFO B display
    } else if ( (p.x > 130) && (p.x < 155) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 5;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 165) && (p.x < 185) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 4;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 190) && (p.x < 210) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 3;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 215) && (p.x < 235) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 2;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 255) && (p.x < 278) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 1;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 275) && (p.x < 295) && (p.y > 65) && (p.y < 100)) {
      Freq_Index = 0;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Place_The_Cursor();


      //  check for freq incr change on VFO A display
    } else if ( (p.x > 130) && (p.x < 155) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 5;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 165) && (p.x < 185) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 4;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 190) && (p.x < 210) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 3;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 215) && (p.x < 235) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 2;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Truncate_Frequency();
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 255) && (p.x < 278) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 1;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Place_The_Cursor();

      //  check for freq incr change
    } else if ( (p.x > 280) && (p.x < 295) && (p.y > 150) && (p.y < 185)) {
      Freq_Index = 0;
      Put_SDEEPROM( FREQ_INCR, Freq_Index );
      Place_The_Cursor();

      // Band
    } else if ( (p.x > 85) && (p.x < 144) && (p.y > 46) && (p.y < 67)) {
      tft.drawRect(2, 46, 185, 22, ILI9341_WHITE);
      tft.fillRect(3, 47, 183, 20, ILI9341_BLUE);
      tft.setFont(Arial_12_Bold);
      tft.setTextColor(ILI9341_WHITE);
      tft.setCursor(5, 51);
      tft.print((char*)"80 40 30 20 17 15 12 10");
      while (ts.touched()) delay(10);
      while (!ts.touched()) delay(10);
      Get_Touch(false);
      if (      (p.x >=  2) && (p.x < 25) && (p.y > 46) && (p.y < 67)) Current_Band = 0;
      else if ( (p.x >= 25) && (p.x < 48) && (p.y > 46) && (p.y < 67)) Current_Band = 1;
      else if ( (p.x >= 50) && (p.x < 73) && (p.y > 46) && (p.y < 67)) Current_Band = 2;
      else if ( (p.x >= 75) && (p.x < 98) && (p.y > 46) && (p.y < 67)) Current_Band = 3;
      else if ( (p.x >= 100) && (p.x < 123) && (p.y > 46) && (p.y < 67)) Current_Band = 4;
      else if ( (p.x >= 125) && (p.x < 148) && (p.y > 46) && (p.y < 67)) Current_Band = 5;
      else if ( (p.x >= 150) && (p.x < 173) && (p.y > 46) && (p.y < 67)) Current_Band = 6;
      else if ( (p.x >= 175) && (p.x < 186) && (p.y > 46) && (p.y < 67)) Current_Band = 7;

      tft.fillRect(2, 46, 185, 22, ILI9341_BLACK);
      tft.drawRect(67, 47, 95, 21,  ILI9341_WHITE);
      tft.fillRect(68, 48, 93, 19,  ILI9341_BLUE);
      tft.setTextColor(ILI9341_WHITE);
      tft.setFont(Arial_12_Bold);
      tft.setCursor(93, 52 );
      tft.println("Band");

      Display_Trangle(70, 51, 16, 13, ILI9341_GREEN);
      Display_Trangle(140, 64, 16, -13, ILI9341_GREEN);
      Set_Band();
      while (ts.touched()) delay(10);


      // Band DN
    } else if ( (p.x > 66) && (p.x < 84) && (p.y > 46) && (p.y < 67)) {
      Current_Band = Current_Band - 1;
      if ( Current_Band < 0 )Current_Band = 0;
      Set_Band();
      while (ts.touched()) delay(10);

      // Band Up
    } else if ( (p.x > 145) && (p.x < 162) && (p.y > 46) && (p.y < 67)) {
      Current_Band = Current_Band + 1;
      if ( Current_Band > 7 )Current_Band = 7;
      Set_Band();
      while (ts.touched()) delay(10);

      // Mode
    } else if ( (p.x > 190) && (p.x < 265) && (p.y > 30) && (p.y < 55)) {
      while (ts.touched()) delay(10);
      Display_RIT();
      Display_Mode_Menu();
      Remove_Mode_Menu();
      if ((mouse_x > 196) && (mouse_x < 254) && (mouse_y > 36) && (mouse_y < 46))tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
      Display_Mouse();

      // RIT
    } else if ( (p.x > 95) && (p.x < 135) && (p.y > 200) && (p.y < 230)) {
      if ( splitOn == 0 ) {
        if (ritOn)Display_RIT();
        else Display_RIT_ON();
        if ( (mouse_x > 93) && (mouse_x < 123) && (mouse_y > 195) && (mouse_y < 208)) Renew_Mouse();
        while ( ts.touched()) delay(10);
      }

      // SPL
    } else if ( (p.x > 50) && (p.x < 85) && (p.y > 200) && (p.y < 230)) {
      if ( !ritOn) menuSplitToggle();
      if ( (mouse_x > 48) && (mouse_x < 80) && (mouse_y > 195) && (mouse_y < 208)) Renew_Mouse();
      while ( ts.touched()) delay(10);

      // Lock
    } else if ( (p.x > 6) && (p.x < 40) && (p.y > 200) && (p.y < 230)) {
      if ( Tune_Lock == 0 ) {
        Display_Lock_ON();
      } else {
        Display_Lock();
      }
      if ( (mouse_x > 0) && (mouse_x < 36) && (mouse_y > 195) && (mouse_y < 208)) Renew_Mouse();
      while ( ts.touched()) delay(10);

      // Table
    } else if ( (p.x > 8) && (p.x < 40) && (p.y > 0) && (p.y < 20)) {
      while ( ts.touched()) delay(10);
      Display_Table_Menu();
      while (!ts.touched()) delay(10);
      Get_Touch();
      Remove_Table_Menu();

      // Channel
    } else if ( (p.x > 60) && (p.x < 75) && (p.y > 0) && (p.y < 20)) {
      while ( ts.touched()) delay(10);
      Display_Channel_Menu();
      while (!ts.touched()) delay(10);
      Get_Touch(false);
      Remove_Channel_Menu();

      ubitx_mode = Working_Copy_Modes[Mem_Number];
      if (vfoActive == VFO_B)vfoB = Working_Copy_Frequencies[Mem_Number];
      else vfoA = Working_Copy_Frequencies[Mem_Number];
      frequency = Working_Copy_Frequencies[Mem_Number];
      Put_SDEEPROM( MODE, ubitx_mode );
      delay(200);
      setFrequency(frequency);
      Display_Mem_Selection();
      Display_Mode();
      updateDisplay();


      // Load
    } else if ( (p.x > 90) && (p.x < 110) && (p.y > 0) && (p.y < 20)) {
      Process_Memory_Load();

      // Store
    } else if ( (p.x > 125) && (p.x < 150) && (p.y > 0) && (p.y < 20)) {
      Process_Memory_Store();

      // CWspeed
    } else if ( (p.x > 6) && (p.x < 45) && (p.y > 40) && (p.y < 55)) {
      menuCWSpeed();

      // Setup
    } else if ( (p.x > 140) && (p.x < 180) && (p.y > 200) && (p.y < 230)) {
      while ( ts.touched() )delay(10);
      Process_Setup();
    }
  }

}

//////////////////////////////////////////////////////////////////////////////////////////

void Remember_Mouse( void) {

  if ( Mouse_Active ) {
    tft.readRect(last_mouse_x, last_mouse_y, 12, 12, Mouse_Data);
  }
}

void Save_Mouse( void) {

  if ( Mouse_Active ) {
    tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
    tft.fillCircle(mouse_x + 5, mouse_y + 5, 4, ILI9341_YELLOW);
  }
}


void Reset_Mouse( void) {
  if ( Mouse_Active ) {
    mouse_x = 10;
    mouse_y = 100;
    last_mouse_x = 10;
    last_mouse_y = 100;
    tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
    tft.fillCircle(mouse_x + 5, mouse_y + 5, 4, ILI9341_YELLOW);
  }
}

void Renew_Mouse( void) {
  if ( Mouse_Active ) {
    tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
  }
}


void Remove_Mouse( void) {
  if ( Mouse_Active ) tft.writeRect(last_mouse_x, last_mouse_y, 12, 12, Mouse_Data);
}



void   Get_Mouse_Data( void ) {
  int local_buttons;
  int local_valx;
  int local_valy;

  Current_Mouse_Wheel = 0;


  if (uBITX_Mouse.available()) {
    mouse_left_button = 0;
    mouse_right_button = 0;

    Current_Mouse_Wheel = uBITX_Mouse.getWheel();
    local_buttons = uBITX_Mouse.getButtons();
    local_valy = uBITX_Mouse.getMouseY();
    local_valx = uBITX_Mouse.getMouseX();
    mouse_left_button = local_buttons & 1;
    mouse_right_button = local_buttons & 2;
    uBITX_Mouse.mouseDataClear();

    mouse_left_button = local_buttons & 1;
    mouse_right_button = local_buttons & 2;

    if ( (local_buttons == 0) && (Current_Mouse_Wheel == 0) ) {
      // if( local_valx > size )local_valx = size;
      // else local_valx = local_valx - 2;
      // if( local_valx < -size )local_valx = -size;
      // else local_valx = local_valx + 2;
      // if( (local_valx > -2)&&(local_valx < 2) )local_valx = 0;
      mouse_x = mouse_x + local_valx * 3 / 4;
      if ( mouse_x > 306) mouse_x = 306;
      if ( mouse_x < 0) mouse_x = 0;
      // if( local_valy > size )local_valy = size;
      // else local_valy = local_valy - 2;
      // if( local_valy < -size )local_valy = -size;
      // else local_valy = local_valy + 2;
      // if( (local_valy > -2)&&(local_valy < 2) )local_valy = 0;
      mouse_y = mouse_y + local_valy * 3 / 4;
      if ( mouse_y > 226) mouse_y = 226;
      if ( mouse_y < 0) mouse_y = 0;

      if ((last_mouse_x != mouse_x) || (last_mouse_y != mouse_y)) {
        tft.writeRect(last_mouse_x, last_mouse_y, 12, 12, Mouse_Data);
        tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
        tft.fillCircle(mouse_x + 5, mouse_y + 5, 4, ILI9341_YELLOW);
        p.x = mouse_x;
        p.y = mouse_y;
        last_mouse_x = mouse_x;
        last_mouse_y = mouse_y;
      }
    }

#ifdef DEBUG
    Serial.print("Mouse: left = ");
    Serial.print(mouse_left_button);
    Serial.print(", right = ");
    Serial.print(mouse_right_button);
    Serial.print(",  mouseX = ");
    Serial.print(mouse_x);
    Serial.print(",  mouseY = ");
    Serial.print(mouse_y);
    Serial.print(",  wheel = ");
    Serial.print(Current_Mouse_Wheel);
    Serial.print(",  wheelH = ");
    Serial.print(uBITX_Mouse.getWheelH());
    Serial.println();
#endif
  }
  delay(5);

}

//void   Get_Mouse_Data( void ) {
// Get_Mouse_Data( 80 );
//}


void Update_Mouse( void) {

  if ( Mouse_Active ) {
    tft.readRect(mouse_x, mouse_y, 12, 12, Mouse_Data);
    tft.fillCircle(mouse_x + 5, mouse_y + 5, 4, ILI9341_YELLOW);
  }
}

int Get_Mouse_Wheel_Value( void ) {

  Get_Mouse_Data();

  return (Current_Mouse_Wheel);
}


bool Get_Left_Button_Pressed( void ) {

  Get_Mouse_Data();

  return (mouse_left_button);
}

bool Set_Left_Button_Pressed( void ) {

  Get_Mouse_Data();

  return (mouse_left_button);
}

void Process_Wheel_Frequency_Update_VFOA( void ) {
  long rx_freq;
  int knob;

  if ( Mouse_Active ) {

    if ( (p.x > 95) && (p.x < 135) && (p.y > 190) && (p.y < 210) ) {
      knob = Get_Mouse_Wheel_Value();
      if (knob != 0 ) {
        if (knob < 0) {
          ritOffset -= 100l;
          if ( abs(ritOffset) > ritOffsetMax) ritOffset = -ritOffsetMax;
        } else if (knob > 0) {
          ritOffset += 100;
          if ( ritOffset > ritOffsetMax) ritOffset = ritOffsetMax;
        }
        setFrequency(frequency);
        updateDisplay();
      }
    }
    if ( vfoActive == VFO_A ) {
      if (Current_Mouse_Wheel == 1) {
        rx_freq = vfoA;
        rx_freq += Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;
        vfoA = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        Update_Mouse();
        delay(100);
      }
      if ( Current_Mouse_Wheel == -1)  {
        rx_freq = vfoA;
        rx_freq -= Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;
        vfoA = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        Update_Mouse();
        delay(100);
      }
    }
  }
}

void Process_Wheel_Frequency_Update_VFOB( void ) {
  long rx_freq;

  if ( Mouse_Active ) {
    if ( vfoActive == VFO_B ) {
      if (Current_Mouse_Wheel == 1) {
        rx_freq = vfoB;
        rx_freq += Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;
        vfoB = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        Update_Mouse();
        delay(100);
      }
      if ( Current_Mouse_Wheel == -1)  {
        rx_freq = vfoB;
        rx_freq -= Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;
        vfoB = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        Update_Mouse();
        delay(100);
      }
    }
  }
}



void Process_Mouse( void ) {
  long rx_freq;

  if ( Mouse_Active ) {
    Get_Mouse_Data();

    if ( PBT_ON && (mouse_right_button != 0 )) {
      if ( (p.x > 7) && (p.x < 40) && (p.y > 120) && (p.y < 139)) {
        PBT_Value = PBT_Value + 20;
        if ( PBT_Value > 2000) PBT_Value = 2000;
        setFrequency(frequency);
        Display_PBT_Position();
        Update_Mouse();
      }
    }


    if ( PBT_ON && (mouse_left_button != 0 )) {
      if ( (p.x > 7) && (p.x < 40) && (p.y > 120) && (p.y < 139)) {
        PBT_Value = PBT_Value - 20;
        if ( PBT_Value <  -2000) PBT_Value =  -2000;
        setFrequency(frequency);
        Display_PBT_Position();
        Update_Mouse();
      }
    }


    if ( PBT_ON && (Current_Mouse_Wheel != 0 )) {
      if ( (p.x > 7) && (p.x < 40) && (p.y > 120) && (p.y < 139)) {
        if (Current_Mouse_Wheel == 1 ) {
          PBT_Value = PBT_Value + 20;
          if ( PBT_Value > 2000) PBT_Value = 2000;
          setFrequency(frequency);
          Display_PBT_Position();
          Update_Mouse();

        } else if (Current_Mouse_Wheel == -1 ) {
          PBT_Value = PBT_Value - 20;
          if ( PBT_Value <  -2000) PBT_Value =  -2000;
          setFrequency(frequency);
          Display_PBT_Position();
          Update_Mouse();

        }
        while (Get_Mouse_Wheel_Value() != 0 ) {
          if (Current_Mouse_Wheel == 1 ) {
            PBT_Value = PBT_Value + 20;
            if ( PBT_Value > 2000) PBT_Value = 2000;
            setFrequency(frequency);
            Display_PBT_Position();
            Update_Mouse();

          } else if (Current_Mouse_Wheel == -1 ) {
            PBT_Value = PBT_Value - 20;
            if ( PBT_Value <  -2000) PBT_Value =  -2000;
            setFrequency(frequency);
            Display_PBT_Position();
            Update_Mouse();

          }
        }
      }
    }

    if ((mouse_left_button == 1) || (Current_Mouse_Wheel != 0)) {
      if ( (p.x > 305) && (p.x < 320) && (p.y > 75) && (p.y < 125)) {
        if ( vfoActive == VFO_B ) rx_freq = vfoB;
        else rx_freq = vfoA;
        rx_freq += Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;
        if ( vfoActive == VFO_B )vfoB = rx_freq;
        else vfoA = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        if (  first == true) {
          delay(1000);
          first = false;
        } else delay(200);


      } else if ( (p.x > 305) && (p.x < 320) && (p.y > 145) && (p.y < 195)) {
        if ( vfoActive == VFO_B ) rx_freq = vfoB;
        else rx_freq = vfoA;
        rx_freq -= Freq_Incr[Freq_Index];
        if ( rx_freq <  400000l) rx_freq =  400000l;
        if ( rx_freq > 30000000l) rx_freq = 30000000l;

        if ( vfoActive == VFO_B )vfoB = rx_freq;
        else vfoA = rx_freq;
        frequency = rx_freq;
        setFrequency(frequency);
        updateDisplay();
        if (  first == true) {
          delay(1000);
          first = false;
        } else delay(200);

        // vfo a
      } else if ( (p.x > 65) && (p.x < 90) && (p.y > 150) && (p.y < 180) && (mouse_left_button == 1)) {
        if (ritOn)Display_RIT();
        if ( vfoActive == VFO_B ) {
          vfoActive = VFO_B;
          menuVfoToggle();
          Display_Mode();
          if ( frequency <=        4000000 )Current_Band = 0;
          else if ( frequency <=   7300000 )Current_Band = 1;
          else if ( frequency <=  10150000 )Current_Band = 2;
          else if ( frequency <=  14350000 )Current_Band = 3;
          else if ( frequency <=  18168000 )Current_Band = 4;
          else if ( frequency <=  21450000 )Current_Band = 5;
          else if ( frequency <=  24990000 )Current_Band = 6;
          else if (frequency <=   29700000 )Current_Band = 7;
          Update_Mouse();
          while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        }
        //        PBT_Active = false;

        // vfo b
      } else if ( (p.x > 65) && (p.x < 90) && (p.y > 70) && (p.y < 100) && (mouse_left_button == 1)) {
        if (ritOn)Display_RIT();
        if ( vfoActive == VFO_A ) {
          vfoActive = VFO_A;
          menuVfoToggle();
          Display_Mode();
          if ( frequency <=        4000000 )Current_Band = 0;
          else if ( frequency <=   7300000 )Current_Band = 1;
          else if ( frequency <=  10150000 )Current_Band = 2;
          else if ( frequency <=  14350000 )Current_Band = 3;
          else if ( frequency <=  18168000 )Current_Band = 4;
          else if ( frequency <=  21450000 )Current_Band = 5;
          else if ( frequency <=  24990000 )Current_Band = 6;
          else if (frequency <=   29700000 )Current_Band = 7;
          Update_Mouse();
          while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        }
        //        PBT_Active = false;

        //  check for freq incr change on VFO B display
      } else if ( (p.x > 130) && (p.x < 155) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 5;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();
        //  check for freq incr change
      } else if ( (p.x > 165) && (p.x < 185) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 4;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();

        //  check for freq incr change
      } else if ( (p.x > 190) && (p.x < 210) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 3;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();

        //  check for freq incr change
      } else if ( (p.x > 215) && (p.x < 235) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 2;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();

        //  check for freq incr change
      } else if ( (p.x > 255) && (p.x < 278) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 1;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();

        //  check for freq incr change
      } else if ( (p.x > 275) && (p.x < 295) && (p.y > 75) && (p.y < 95)) {
        Freq_Index = 0;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOB();


        //  check for freq incr change on VFO A display
      } else if ( (p.x > 130) && (p.x < 155) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 5;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        //  check for freq incr change
      } else if ( (p.x > 165) && (p.x < 185) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 4;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        //  check for freq incr change
      } else if ( (p.x > 190) && (p.x < 210) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 3;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        //  check for freq incr change
      } else if ( (p.x > 215) && (p.x < 235) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 2;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Truncate_Frequency();
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        //  check for freq incr change
      } else if ( (p.x > 255) && (p.x < 278) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 1;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        //  check for freq incr change
      } else if ( (p.x > 280) && (p.x < 295) && (p.y > 150) && (p.y < 175)) {
        Freq_Index = 0;
        Put_SDEEPROM( FREQ_INCR, Freq_Index );
        Place_The_Cursor();
        Process_Wheel_Frequency_Update_VFOA();

        // Band
      } else if ( (p.x > 94) && (p.x < 123) && (p.y > 46) && (p.y < 56) && (Set_Left_Button_Pressed() == 1)) {
        tft.drawRect(2, 46, 185, 22, ILI9341_WHITE);
        tft.fillRect(3, 47, 183, 20, ILI9341_BLUE);
        tft.setFont(Arial_12_Bold);
        tft.setTextColor(ILI9341_WHITE);
        tft.setCursor(5, 51);
        tft.print((char*)"80 40 30 20 17 15 12 10");
        Update_Mouse();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay();
        while (Set_Left_Button_Pressed() == 0) Fake_Delay();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay();
        if (      (p.x >=  2) && (p.x < 25) && (p.y > 46) && (p.y < 56)) Current_Band = 0;
        else if ( (p.x >= 25) && (p.x < 48) && (p.y > 46) && (p.y < 56)) Current_Band = 1;
        else if ( (p.x >= 50) && (p.x < 73) && (p.y > 46) && (p.y < 56)) Current_Band = 2;
        else if ( (p.x >= 75) && (p.x < 98) && (p.y > 46) && (p.y < 56)) Current_Band = 3;
        else if ( (p.x >= 100) && (p.x < 123) && (p.y > 46) && (p.y < 56)) Current_Band = 4;
        else if ( (p.x >= 125) && (p.x < 148) && (p.y > 46) && (p.y < 56)) Current_Band = 5;
        else if ( (p.x >= 145) && (p.x < 152) && (p.y > 46) && (p.y < 56)) Current_Band = 6;
        else if ( (p.x >= 170) && (p.x < 176) && (p.y > 46) && (p.y < 56)) Current_Band = 7;


        tft.fillRect(2, 46, 185, 22, ILI9341_BLACK);
        tft.drawRect(67, 47, 95, 21,  ILI9341_WHITE);
        tft.fillRect(68, 48, 93, 19,  ILI9341_BLUE);
        tft.setTextColor(ILI9341_WHITE);
        tft.setFont(Arial_12_Bold);
        tft.setCursor(93, 52 );
        tft.println("Band");
        Display_Trangle(70, 51, 16, 13, ILI9341_GREEN);
        Display_Trangle(140, 64, 16, -13, ILI9341_GREEN);

        Set_Band();
        delay(20);

        while (Set_Left_Button_Pressed() == 1) Fake_Delay();
        Update_Mouse();

        // Band DN
      } else if ( (p.x > 66) && (p.x < 80) && (p.y > 50) && (p.y < 56) && (mouse_left_button == 1)) {
        Current_Band = Current_Band - 1;
        if ( Current_Band < 0 )Current_Band = 0;
        Set_Band();
        Save_Mouse();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();

        // Band Up
      } else if ( (p.x > 140) && (p.x < 150) && (p.y > 50) && (p.y < 56) && (mouse_left_button == 1)) {
        Current_Band = Current_Band + 1;
        if ( Current_Band > 7 )Current_Band = 7;
        Set_Band();
        Save_Mouse();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();

        // Mode
      } else if ( (p.x > 200) && (p.x < 265) && (p.y > 30) && (p.y < 55) && (mouse_left_button == 1)) {
        while ( Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Display_RIT();
        Display_Mode_Menu();
        Update_Mouse();
        Remove_Mode_Menu();
        Update_Mouse();


        // RIT
      } else if ( (p.x > 95) && (p.x < 135) && (p.y > 190) && (p.y < 210) && (mouse_left_button == 1)) {
        //Update_Mouse();
        if ( splitOn == 0 ) {
          if (ritOn) {
            Display_RIT();
            Update_Mouse();
          } else {
            Display_RIT_ON();
            Update_Mouse();
          }
          while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        }

        // SPL
      } else if ( (p.x > 50) && (p.x < 85) && (p.y > 190) && (p.y < 210) && (mouse_left_button == 1)) {
        if ( !ritOn) {
          menuSplitToggle();
          Update_Mouse();
        }
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();

        // Lock
      } else if ( (p.x > 6) && (p.x < 40) && (p.y > 190) && (p.y < 210) && (mouse_left_button == 1)) {
        if ( Tune_Lock == 0 ) {
          Display_Lock_ON();
        } else {
          Display_Lock();
        }
        Update_Mouse();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();

        // Table
      } else if ( (p.x > 8) && (p.x < 40) && (p.y > 0) && (p.y < 20) && (mouse_left_button == 1)) {
        Display_Table_Menu();
        Update_Mouse();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay();
        while (Set_Left_Button_Pressed() == 0) Fake_Delay();
        Remove_Table_Menu();
        while (Set_Left_Button_Pressed() == 1) Fake_Delay();
        if ( (p.x > 2) && (p.x < 54) && (p.y > 2) && (p.y < 80))  Update_Mouse();

        // Channel
      } else if ( (p.x > 60) && (p.x < 75) && (p.y > 0) && (p.y < 20) && (mouse_left_button == 1)) {
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Display_Channel_Menu();
        Update_Mouse();
        while (Set_Left_Button_Pressed() == 0) Fake_Delay1();
        Remove_Channel_Menu();
        if ( (p.x > 2) && (p.x < 180) && (p.y > 2) && (p.y < 230))  Update_Mouse();
        ubitx_mode = Working_Copy_Modes[Mem_Number];
        if (vfoActive == VFO_B)vfoB = Working_Copy_Frequencies[Mem_Number];
        else vfoA = Working_Copy_Frequencies[Mem_Number];
        frequency = Working_Copy_Frequencies[Mem_Number];
        Put_SDEEPROM( MODE, ubitx_mode );
        delay(200);
        setFrequency(frequency);
        Display_Mem_Selection();
        Display_Mode();
        updateDisplay();


        // Load
      } else if ( (p.x > 90) && (p.x < 110) && (p.y > 0) && (p.y < 10) && (mouse_left_button == 1)) {
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Process_Memory_Load();

        // Store
      } else if ( (p.x > 118) && (p.x < 131) && (p.y > 0) && (p.y < 10) && (mouse_left_button == 1)) {
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Process_Memory_Store();

        // CWspeed
      } else if ( (p.x > 24) && (p.x < 42) && (p.y > 45) && (p.y < 56) && (mouse_left_button == 1)) {
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        menuCWSpeed();

        // Setup
      } else if ( (p.x > 140) && (p.x < 180) && (p.y > 190) && (p.y < 210) && (mouse_left_button == 1)) {
        while ( Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Process_Setup();

        // PBT
      } else if ( (p.x > 20) && (p.x < 30) && (p.y > 120) && (p.y < 140) && (mouse_left_button == 1)) {
        if ( PBT_ON ) {
          Display_PBT_OFF();
        } else {
          Display_PBT_Position();
          PBT_ON = true;
          //          PBT_Active = true;
          //          Rit_Encoder = false;
        }
        while (Set_Left_Button_Pressed() == 1) Fake_Delay1();
        Update_Mouse();
      }
    }
  }


}
